/*===========================================================================*/
/* SEED reader     |              times_data               |    subprocedure */
/*===========================================================================*/
/*
	Name:		times_data
	Purpose:	determine and print station, channel, start time, number of 
			samples, sample rate, and station/channel/time continutity flag
			for each data record on a SEED volume
	Usage:		void times_data ();
				times_data ();
	Input:		none
	Output:		none (prints information to stdout)
	Externals:	input.data - pointer to beginning of data block
	Warnings:	none
	Errors:		none
	Called by:	snoop
	Calls to:	none
	Algorithm:	determine whether this record is time/station/channel
			continuous with the previous record; print the record number,
			station name, channel name, start time of record, and the
			continuity flag
	Notes:		A continuity flag of 0 (FALSE) means that the record is 
			time/station/channel discontinuous with the previous record,
			while a flag of 1 (TRUE) indicates continuity.  
			A time/station/channel discontinuity means that a new
			seismogram has started.
			For ease of use with "grep", the symbol "<==" is appended to 
			lines indicating beginnings of seismograms.
	Problems:	none known
	References:	Halbert et al, 1988; see main routine
	Language:	C, hopefully ANSI standard
	Author:		Dennis O'Neill
	Revisions:	11/21/88  Dennis O'Neill  Initial version
				11/21/88  Dennis O'Neill  Production release 1.0
*/

#include "rdseed.h"

long int total_samples;

#define DATE_IS_GARBAGE(d) ((d.year < 1960) || (d.year > 2010) || \
	(d.day == 0) || (d.day > 366))

double sample_rate;

static struct time newtime;		/* start time of new seisgm */


struct type50 *get_station_rec();
struct type52 *get_channel_rec();
void do_time_correction();

void dump_station_effective();
void dump_channel_effective();
void swap_fsdh();
void blockette_swap();
void blockette_check();

static int fsdh_swapped = 0;

struct {
	struct input_time start;
	struct input_time end;
	char stn[6];
	char chn[4];
	char loc[3];
	char net[3];
	int ts_started;	

	struct node {
		char *m;
		struct node *next;
	} msg;

} this_ts = {{0, 0, 
		'\0', '\0', '\0', 
		'\0',	/* unused */
		0},
	     {0, 0, 
                '\0', '\0', '\0', 
                '\0',   /* unused */
                0},
   	     "", "", "", "", 
		0,
		{ "", (struct node *)NULL}};

/* ---------------------------------------------------------------------- */

void times_data (call_type)
int call_type;
{
	int i;				/* counter */
	char *input_data_ptr;		/* pointer to input data */
	struct input_data_hdr *input_data_hdr;	/* input data header */

	struct type50	*p_station;/* ptr to current station */
	struct type52	*p_channel;/* ptr to current channel */

	int continuous;			/* continuity flag */
	int duration;			/* length of seismogram */
	int tempint;			/* temporary integer */
	short int temp_2byte;		/* temp for byte swapping */
	unsigned short int temp_u2byte;	/* temp for byte swapping */
	struct input_time temptime;	/* temp for byte swapping */

	char this_station[5+1];		/* name of station */
	char this_channel[3+1];		/* name of channel */
	char this_location[3];		/* location, duh   */
	char this_network[3];
	char tmpstr[256];
	struct time this_time;		/* start time of this seisgm */
static int ack = 0;

	/* figure out current ending time of previous seismogram */
	if (last_sample_rate != 0)
		duration = (int) ((last_nsamples * 10000) / last_sample_rate);
	else 
	{
		duration = 0;
		last_time.year = 0;
		last_time.day = 0;
		last_time.hour = 0;
		last_time.minute = 0;
		last_time.second = 0;
		last_time.fracsec = 0;
	}
	newtime = timeadd (last_time, duration);

	if (!call_type)
	{

/*	input_data_ptr = input.data; */
	input_data_ptr = lrecord_ptr+8;

/*                 +=======================================+                 */
/*=================|   read the fixed part of data header  |=================*/
/*                 +=======================================+                 */

	/* recover the fixed data header from the input record */
	input_data_hdr = (struct input_data_hdr *) input_data_ptr;

	if (input_data_hdr->nsamples == 0)
	{
		return;
	}

/*                 +=======================================+                 */
/*=================| obtain particulars for this data blk  |=================*/
/*                 +=======================================+                 */

	/* recover station name */
	for (i = 0; i < 5; i++) 
		this_station[i] = input_data_hdr->station[i];

	this_station[5] = '\0';

	for (i = 4; this_station[i] == ' '; i--) 
		this_station[i] = '\0';

	/* recover channel name */
	for (i = 0; i < 3; i++) 
		this_channel[i] = input_data_hdr->channel[i];

	this_channel[3] = '\0';

	for (i = 2; this_channel[i] == ' '; i--) 
		this_channel[i] = '\0';

	       /* skip if a LOG record */
        if (strcasecmp(this_channel, "LOG") == 0)
                return;


	/* recover the network code */
    	if (type10.version >= 2.3)
    	{
        	char *p;
 
        	memcpy(this_network, input_data_hdr->network, 2);
        	this_network[2] = 0;
 
        	if ((p = strchr(this_network, ' ')) != NULL)
            		*p = 0;
    
    	}
	else
		strcpy(this_network, "");

	/* location code */
        for (i = 0; i < 2; i++)
                this_location[i] = input_data_hdr->location[i];

        this_location[2] = '\0';
 
        for (i = 1; this_location[i] == ' '; i--)
        {
                this_location[i] = '\0';
                if (i == 0)
                        break;

        }

	current_channel = (struct type52 *)NULL;
	current_station = (struct type50 *)NULL;

   	/* check if byteswapping will be needed to recover data */
	if (DATE_IS_GARBAGE(input_data_hdr->time))
        {
                swap_fsdh(&input_data_hdr, &input_data_ptr);
                fsdh_swapped = 1;
 
       } else {
		if (!fsdh_swapped && input_data_hdr->bofb) {
               		 blockette_check((input_data_ptr + input_data_hdr->bofb - 8),
			input_data_ptr - 8);
}
	}
 
        /* check for sanity */
	if (DATE_IS_GARBAGE(input_data_hdr->time))
        {
                fprintf(stderr, "ERROR - process_data(): Unknown word order for station %s, channel %s, network %s, location %s\n",
                                this_station,
                                this_channel,
                                this_network,
                                this_location);
 
                fprintf(stderr, "Skipping data record.\n");
                return;
        }

	/* get start time of this data block */
       	temptime.year    = input_data_hdr->time.year;
        temptime.day     = input_data_hdr->time.day;
        temptime.hour    = input_data_hdr->time.hour;
        temptime.minute  = input_data_hdr->time.minute;
        temptime.second  = input_data_hdr->time.second;
        temptime.fracsec = input_data_hdr->time.fracsec;
 
        this_time.year    = (int) temptime.year;
        this_time.day     = (int) temptime.day;
        this_time.hour    = (int) temptime.hour;
        this_time.minute  = (int) temptime.minute;
        this_time.second  = (int) temptime.second;
        this_time.fracsec = (int) temptime.fracsec;
 


	/* Do time correction */

	if ((input_data_hdr->activity_flags & 0x02) == 0)
        {
		this_time.fracsec += input_data_hdr->number_time_corrections;
 
		do_time_correction(&this_time);
 
        }

        get_stn_chn_rec(this_station,
                        this_channel,
                        this_network,
                        this_location,
                        &this_time);
 
        if (current_station == NULL)
        {
		log_tspan_error(this_station, 
				this_channel,
				this_network,
				this_location,
				&input_data_hdr->time,
				"ERROR: Station missing from station table");

		if (sniff_flag > 0)
		{
			fprintf(stderr, "ERROR: Station %5.5s missing from blockette 50s\n", this_station);

			fprintf(stderr, " for stn/chn/net/loc: %s, %s, %s, %s\n", 
					this_station, 
					this_channel, 
					this_network,
					this_location);

			fprintf(stderr, " at: %d, %d, %d:%d:%d.%d\n",
					input_data_hdr->time.year,
					input_data_hdr->time.day,
					input_data_hdr->time.hour,
					input_data_hdr->time.minute,
					input_data_hdr->time.second,
					input_data_hdr->time.fracsec);

		}	/* if sniff_flag > 0 */

                return;
        }
 
        if (current_channel == (struct type52 *)NULL)
        {
                log_tspan_error(this_station,
				this_channel,
				this_network,
				this_location,
				&input_data_hdr->time,
				"ERROR: Channel missing from channel table");

		if (sniff_flag > 0)
		{
			fprintf(stderr, "ERROR: Channel %s missing from blockette 52 for station %s\n", this_channel, this_station);

			fprintf(stderr, " for stn/chn/net/loc: %s, %s, %s, %s\n", 
					this_station, 
					this_channel, 
					this_network,
					this_location);

			fprintf(stderr, " time: %d, %d, %d:%d:%d.%d\n",
					input_data_hdr->time.year,
					input_data_hdr->time.day,
					input_data_hdr->time.hour,
					input_data_hdr->time.minute,
					input_data_hdr->time.second,
					input_data_hdr->time.fracsec);

		}

                return;
        }
 
        byteswap = find_wordorder ();
 
        if (fsdh_swapped && (byteswap == 0))
        {
		log_tspan_error(this_station,
				this_channel,
				this_network,
				this_location,
				&input_data_hdr->time,
				"WARNING: Data is swapped, but stn header indicates no swapping needed");

		if (sniff_flag > 0)
		{
			fprintf(stderr, "WARNING: data is swapped, but stn header indicates no swapping needed\n");

			fprintf(stderr, " for stn/chn/net/loc: %s, %s, %s, %s\n", 
					this_station, 
					this_channel, 
					this_network,
					this_location);

			fprintf(stderr, " time: %d, %d, %d:%d:%d.%d\n",
					input_data_hdr->time.year,
					input_data_hdr->time.day,
					input_data_hdr->time.hour,
					input_data_hdr->time.minute,
					input_data_hdr->time.second,
					input_data_hdr->time.fracsec);

		}

        }
 

/*                 +=======================================+                 */
/*=================| accumulate various descriptive info   |=================*/
/*                 +=======================================+                 */


/*                 +=======================================+                 */
/*=================|          determine continuity         |=================*/
/*                 +=======================================+                 */

	/* determine whether this is station-, channel-, and time-continuous */
	/* with previous data block */


	if ((strcmp(data_hdr->station, this_station) == 0) &&
		(strcmp(data_hdr->channel, this_channel) == 0) &&
		(strcmp(data_hdr->location, this_location) == 0) &&
		(strcmp(data_hdr->network, this_network) == 0) &&
		(timetol(newtime, this_time, last_sample_rate) == 0))
		continuous = TRUE;
	else continuous = FALSE;

	/* save information for next time around (continuity check) */

	strcpy(data_hdr->station, this_station);
	strcpy(data_hdr->channel, this_channel);
	strcpy(data_hdr->location, this_location);
	strcpy(data_hdr->network, this_network);

	last_time.year    = (int) this_time.year;
	last_time.day     = (int) this_time.day;
	last_time.hour    = (int) this_time.hour;
	last_time.minute  = (int) this_time.minute;
	last_time.second  = (int) this_time.second;
	last_time.fracsec = (int) this_time.fracsec;
	temp_u2byte        = input_data_hdr->nsamples;
	last_nsamples     = (int) input_data_hdr->nsamples;

	/* get sample rate */
	last_sample_rate = (double)input_data_hdr->sample_rate;

	tempint = (int) input_data_hdr->sample_rate_multiplier;

	if (last_sample_rate < 0) 
		last_sample_rate = 1/(-last_sample_rate);
	if (tempint > 0) 
		last_sample_rate = last_sample_rate * tempint;
	else if (tempint < 0) 
		last_sample_rate = last_sample_rate / (-tempint);

	if ((int)((last_sample_rate+.005)*100) != 
			(int)((current_channel->samplerate+.005)*100))
	{

	sprintf(tmpstr, "WARNING (process_data): Data sample rate(%f) != Blk52 (%f)",
			last_sample_rate, current_channel->samplerate);

		log_tspan_error(this_station,
				this_channel,
				this_network,
				this_location,
				&input_data_hdr->time,tmpstr);

		if (sniff_flag > 0)
		{
			/*fprintf(stderr, "%s\n", tmpstr);*/

		fprintf(stderr, "WARNING (process_data): Data sample rate(%f) != Blk52 (%f)",
			last_sample_rate, current_channel->samplerate);

			fprintf(stderr, " for stn/chn/net/loc: %s, %s, %s, %s", 
					this_station, 
					this_channel, 
					this_network,
					this_location);

			fprintf(stderr, " time: %d, %d, %d:%d:%d.%d\n",
					input_data_hdr->time.year,
					input_data_hdr->time.day,
					input_data_hdr->time.hour,
					input_data_hdr->time.minute,
					input_data_hdr->time.second,
					input_data_hdr->time.fracsec);

		}
	}


	parse_type100(input_data_ptr, &last_sample_rate);

	}
	else
		continuous = FALSE;
	
	if (continuous)
	{
		total_samples += (long int) last_nsamples;
	}
	else /* if (!(continuous) && (this_time.year != 0)) */
	{
		if (start_record != 0)
		{
			if (sample_rate_count) sample_rate_accum = sample_rate_accum/sample_rate_count;
			else sample_rate_accum = sample_rate;

			if (sniff_flag >= 2)
			{
				fprintf (stderr,"%4d,%03d,%02d:%02d:%02d.%04d", 
					newtime.year, newtime.day, newtime.hour, newtime.minute,
					newtime.second, newtime.fracsec);
				fprintf (stderr,"%11.7f ", sample_rate);
/*				fprintf (stderr,"%11.7f ", sample_rate_accum); */
				fprintf (stderr,"%6d", total_samples);
				fprintf (stderr,"\n");
			}
			sample_rate_accum = 0.0;
			sample_rate_count = 0;
			number_of_time_series++;
		}

/*** Added by Sue to fix (I hope) incorrect display of record numbers ***/
		if (!call_type)
		{
			start_record = input.recordnumber;

			if (sniff_flag >= 2)
			{
				fprintf (stderr,"%6ld ", start_record);
				fprintf (stderr,"%-5s ", this_station);
				fprintf (stderr,"%-3s ", this_channel);
				fprintf (stderr,"%4d,%03d,%02d:%02d:%02d.%04d\n", 
					this_time.year, this_time.day, this_time.hour, this_time.minute,
					this_time.second, this_time.fracsec);
			}

			total_samples = last_nsamples;
			start_record = input.recordnumber;
			sample_rate = last_sample_rate;
		}
	}
}

/* --------------------------------------------------------------------- */
/* --------------------------------------------------------------------- */
#define WITHIN(t, s, e) ((timecmp(t, s) >= 0) && (timecmp(t, e) <= 0))

/* --------------------------------------------------------------------- */
struct type50 *find_station_rec(stn, net_code, beg_time, end_time)
char *stn, *net_code;
struct time *beg_time;
struct time *end_time;
 
{
    struct type50 *p, *p2 = NULL;
    int found;
 
    struct time start,
                end;        /* response effective times */
 
    found = FALSE;
 
    if (end_time->year == 0)
        end_time->year = 9999;
 
    for (p = type50_head; p != NULL;p = p->next)
    {
        if (strcmp(p->station, stn) != 0)
            continue;
 
        if (type10.version >= 2.3)
        {
            /* check to make sure there is network code,
             * as blank network code on disk, turns out as a NULL string
             */
            if (p->network_code)
                if (strcmp( p->network_code, net_code) != 0)
                    continue;
        }
 
 
        /* convert station time (YYYY,DDD,....) to struct */
        timecvt(&start, p->start);
        timecvt(&end, p->end);
 
        if (p->end == NULL)
            end.year = 9999;
 
        /* do time check */
        if ((WITHIN((*beg_time), start, end)) &&
            (WITHIN((*end_time), start, end)))
                p2 = p;
    }
 
    return p2;
 
}

/* ------------------------------------------------------------------------ */
struct type50 *get_station_rec(s, n, rec_time)
char *s;
char *n;
struct time *rec_time;

{
    struct type50 *p, *p2 = NULL;

    struct time start,
                end;        /* response effective times */


    for (p = type50_head; p != NULL;p = p->next)
    {
        if ((strcmp(s, p->station) == 0) &&
            (type10.version >= 2.3 ? (strcmp(n, p->network_code) == 0) : 1))
        {
            timecvt(&start, p->start);
            timecvt(&end, p->end);

            if (p->end == NULL)
                end.year = 9999;

            /* if no time is given, just use the last one found */
            if (rec_time == NULL)
                p2 = p;
            else
                /* else, do time check */
                if (WITHIN((*rec_time), start, end))
                    p2 = p;

		}
	} 

    return p2;
 
}
/* ------------------------------------------------------------------------ */
void dump_station_effective(s, n)
char *s;		/* station name */
char *n;		/* network */

{
	struct type50 *p = NULL;

	for (p = type50_head; p != NULL;p = p->next)
	{
		if ((strcmp(s, p->station) == 0) &&
			(type10.version >= 2.3 ? (strcmp(n, p->network_code) == 0) : 1))
        	{
			fprintf(stderr, 
				"Station %s, network %s, start/stop times: %s / %s\n",	
				s, type10.version >= 2.3 ? n : "N/A", 
				p->start == NULL ? "No starting time" : 
					p->start, 
				p->end == NULL? "No ending time" : 
					p->end);
		}
	} 

}

/* ------------------------------------------------------------------------ */
void dump_channel_effective(now_chn, which_channel)
struct type52 *now_chn;
char which_channel[];

{
	while (now_chn)
	{
		if (strcmp(now_chn->channel, which_channel) == 0)
		{
			fprintf(stderr, " Channel start = %s end = %s\n",
				now_chn->start == NULL ? "No starting time": 
					now_chn->start, 
				now_chn->end == NULL ? "No ending time" : 
					now_chn->end);
		}

		now_chn = now_chn->next;

	}

}

/* ------------------------------------------------------------------- */
/* ------------------------------------------------------------------- */
struct type52 *get_channel_rec(s, channel, l, rec_time)
struct type50 *s;
char *channel;
char *l;
struct time *rec_time;

{
        struct time start, end;
        struct type52 *c, *p = NULL;

        char station[10], network[3], loc[3];
        int finished = 0;

        int n = 3;

        /*## kludge alert, if multiplexed data, just look at
         * the 1st two chars of channel - return after finding
         * the 1st matching record - the output*.c routines will
         * loop through the next two, adding the last char
         */
 
        n = strlen(channel);
 
        strcpy(loc, l);
 
        /* save original station/network so we know when to
         * stop scanning station list
         */
        strcpy(station, s->station);
        if (type10.version >= 2.3)
                strcpy(network, s->network_code);
        
        do
        {
                for (c = s->type52_head; c != NULL; c = c->next)
                        if ((strncmp (channel, c->channel, n) == 0) &&
                                strcmp(loc, c->location) == 0)
                {
                        timecvt(&start, c->start);
                        timecvt(&end,   c->end);
 
                        if (c->end == NULL)
                                end.year = 9999;
 
                        if (rec_time != NULL)
                        {
                                if (WITHIN((*rec_time), start, end))
                                {
                                        p = c;
 
                                        if (c->subchannel > 0)
                                                return(p);
                                }
                        }
                        else
                        {
                                p = c;
        
                                if (c->subchannel > 0)
                                        return(p);
                        }
 
                }
 
                /* got here without finding channel record, check
                 * the next station record, could be there
                 */
                s = s->next;
                finished = s == 0;
 
        } while (!finished && ((strcmp(s->station, station) == 0) &&
                        (type10.version >= 2.3 ?
                                (strcmp(s->network_code, network) == 0) :
                                        1)));
 
        return p;
 
}

/* --------------------------------------------------------------------- */
/* actually sets the global variable current_station and current_channel
 * to point to the right station and channel
 */

int get_stn_chn_rec(s, c, n, l, t)
char *s, *c, *l, *n;
struct time *t;            /* station, channel, location, net, time */

{

        struct type50 *p, *p2 = NULL;
        struct type52 *c_rec = NULL, *crec2 = NULL;


        struct time start, end;        /* response effective times */

        char stn[10];                   /* if we ever get more than 10
                                         * for station and network sizes,
                                         * we need to upgrade
                                         */
        char net[10];

        strncpy(stn, s, 10);
        strncpy(net, n, 10);
 
        trim(stn);
        trim(net);
 
        /* check to see if a time epoch was requested */
        if (t == NULL)
        {
                /* spin until you find a station record */
                for (p = type50_head; p != NULL;p = p->next)
                {
                        if (strcmp(stn, p->station) != 0)
                                continue;
 
                        if ((type10.version >= 2.3))
                                if (strcmp(net, p->network_code) != 0)
                                        continue;
 
                        p2 = p; /* remember station rec */
 
                        /* spin until you find any channel record */
                        if (p->type52_head != NULL)
                        {
                                /* check channel */
                                c_rec = get_channel_rec(p, c, l, NULL);
 
                                if (c_rec != NULL)
                                        crec2 = c_rec; /* remember channel */
                        }
         
                }       /* for every station */
 
                current_station = p2;
                current_channel = crec2;
 
                return 1;
 
        }               /* if not time is requested */
 
 
        /* else scan based on time */
        for (p = type50_head; p != NULL;p = p->next)
        {
                if (strcmp(stn, p->station) != 0)
                        continue;
 
                if ((type10.version >= 2.3))
                        if (strcmp(net, p->network_code) != 0)
                                continue;
 
                timecvt(&start, p->start);
                timecvt(&end, p->end);
 
                if (p->end == NULL)
                        end.year = 9999;
 
                if (WITHIN((*t), start, end))
                        p2 = p; /* remember station rec */
 
                /* make check for channel records -
                 * it is very possible that the chn records have
                 * been tagged onto a stn rec outside of the
                 * time bounds
                 */
 
                if (p->type52_head != NULL)
                {
 
                        /* check channel */
                        c_rec = get_channel_rec(p, c, l, t);
 
                        if (c_rec != NULL)
                                crec2 = c_rec; /* ditto channel */
                }
        
       
        }
 
        current_station = p2;
        current_channel = crec2;

	return 1;

}


/* --------------------------------------------------------------------- */

void swap_fsdh(input_data_hdr, input_data_ptr)
struct input_data_hdr **input_data_hdr;
char **input_data_ptr;

{
        (*input_data_hdr)->time.year =
                swap_2byte((*input_data_hdr)->time.year);

        (*input_data_hdr)->time.day =
        swap_2byte((*input_data_hdr)->time.day);

        (*input_data_hdr)->time.fracsec =
                swap_2byte((*input_data_hdr)->time.fracsec);

	/* make a check for garbage, as we don't want to continue
	 * to swap if garbage as the data blockettes could cause core dump
	 */

	if (DATE_IS_GARBAGE((*input_data_hdr)->time))
		return;

        (*input_data_hdr)->nsamples =
                swap_2byte((*input_data_hdr)->nsamples);

        (*input_data_hdr)->sample_rate =
                swap_2byte((*input_data_hdr)->sample_rate);

        (*input_data_hdr)->sample_rate_multiplier =
                swap_2byte((*input_data_hdr)->sample_rate_multiplier);
 
        (*input_data_hdr)->number_time_corrections =
                swap_4byte((*input_data_hdr)->number_time_corrections);
 
        (*input_data_hdr)->bod =
                swap_2byte((*input_data_hdr)->bod);
 
        (*input_data_hdr)->bofb =
                swap_2byte((*input_data_hdr)->bofb);
 
        if ((*input_data_hdr)->bofb)
                blockette_swap((*input_data_ptr + (*input_data_hdr)->bofb - 8),
			*input_data_ptr - 8);
 
}

/* ------------------------------------------------------------------ */
void blockette_swap(b_ptr, base)
struct data_blk_hdr *b_ptr;
char *base;


{
        short type, next_blk_byte;

        while (1)
        {
                b_ptr->type = swap_2byte(b_ptr->type);

                b_ptr->next_blk_byte = swap_2byte(b_ptr->next_blk_byte);
         
                switch (b_ptr->type)
                {
                        case 100 :
                                { 
                                char *p;
                                float s_rate;
        
                                /* must switch the sample rate - float */
                                p = (char *)&((struct data_blk_100 *)b_ptr)->sample_rate;
 
                                *((int *)&s_rate)=swap_4byte(*((int *)p));
 
                                ((struct data_blk_100 *)b_ptr)->sample_rate =
                                        s_rate;
 
                                }
 
                                break;
         
                        case 201:
                        case 300:
                        case 310:
                        case 200:
                        case 320:
                        case 390:
                        case 395:
                        case 400:
                        case 405:
                        case 500:
                                break;
                        case 1000:
			case 1001:
                                break;
                        default : /* oh, oh */
 
                                fprintf(stderr,
"blockette swapper: Bad blockette scanned\n Blockette = %d\n", b_ptr->type);
 
                                return;
 
                }               /* switch */
        
                if (b_ptr->next_blk_byte == 0)
                        return;
 
                b_ptr = (struct data_blk_hdr *)(base + b_ptr->next_blk_byte);
 
        }   /* while */
 
}

/* ------------------------------------------------------------------ */
void blockette_check(b_ptr, base)
struct data_blk_hdr *b_ptr;
char *base;


{
        short type, next_blk_byte;

        while (1)
        {
                switch (b_ptr->type)
                {
                        case 100 :
                        case 201:
                        case 300:
                        case 310:
                        case 200:
                        case 320:
                        case 390:
                        case 395:
                        case 400:
                        case 405:
                        case 500:
                        case 1000:
			case 1001:
                                break;
                        default : /* oh, oh */
 
                                fprintf(stderr,
"blockette checker: Bad blockette scanned\n Blockette = %d\n", b_ptr->type);
 
                                return;
 
                }               /* switch */
        
                if (b_ptr->next_blk_byte == 0)
                        return;
 
                b_ptr = (struct data_blk_hdr *)(base + b_ptr->next_blk_byte);
 
        }   /* while */
 
}
 
/* ------------------------------------------------------------------ */
void set_fsdh_swapped_flag(flag)
int flag;

{
	fsdh_swapped = flag;
}


/* ------------------------------------------------------------------- */

void do_time_correction(t)
struct time *t;

{


    while ((t->fracsec >= 10000) || (t->fracsec < 0))
    {
        if (t->fracsec >= 10000)
        {
            t->second += t->fracsec/10000;
            t->fracsec = t->fracsec%10000;

            if (t->second >= 60)
            {
                t->minute += t->second/60;
                t->second = t->second%60;
                if (t->minute >= 60)
                {
                    t->hour += t->minute/60;
                    t->minute = t->minute%60;
                    if (t->hour >= 24)
                    {
                        t->day += t->hour/24;
                        t->hour = t->hour%24;
                        if (t->day > (isaleap(t->year) ? 366 : 365))
                        {
                            t->year += 1;
                            t->day = 1;
                        }
                    }
                }
            }
        }
        else if (t->fracsec < 0)
        {
            t->second -= 1;
            t->fracsec += 10000;
            if (t->second < 0)
            {
                t->minute -= 1;
                t->second += 60;
                if (t->minute < 0)
                {
                    t->hour -= 1;
                    t->minute += 60;
                    if (t->hour < 0)
                    {
                        t->day -= 1;
                        t->hour += 24;
                        if (t->day == 0)
                        {
                            t->year -= 1;
                            t->day = (isaleap(t->year) ? 366 : 365);
                        }
                    }
                }
            }
        }
    }
}
 
/* --------------------------------------------------------------------- */

int log_tspan_error(s, c, n, l, t, err_msg)
char *s, *c, *n, *l;
struct input_time *t;
char *err_msg;

{

	struct node *node;

	int found = 0;
	if (!this_ts.ts_started)
	{
		strcpy(this_ts.stn, s);
		strcpy(this_ts.chn, c);
		strcpy(this_ts.loc, l);
		strcpy(this_ts.net, n);

		this_ts.start = *t;

		this_ts.msg.m = malloc(strlen(err_msg)+1);
		strcpy(this_ts.msg.m, err_msg);

		this_ts.msg.next = NULL;

		this_ts.ts_started = 1;

		return 0;
	}

	/* if new stn/chn/net/loc, output previous error msgs */

	if ((strcmp(this_ts.stn, s) != 0) ||
            (strcmp(this_ts.chn, c) != 0) || 
            (strcmp(this_ts.loc, l) != 0) ||
            (strcmp(this_ts.net, n) != 0))
	{
		print_ts_error();
		reset_ts_struct();

		log_tspan_error(s, c, n, l, t, err_msg);
		this_ts.ts_started = 0;

		return 0;

	}

	/* else log the endtime */
	this_ts.end = *t;

	node = &this_ts.msg;

	do
	{
		if (strcmp(node->m, err_msg) == 0)
			found = 1;

		/* always end at last node, for appending if needed */
		if (node->next == NULL)
			break;
		else
			node = node->next;

	} while (1);

	if (!found)
	{
		node->next = (struct node *)malloc(sizeof(struct node));
		node->next->m = malloc(strlen(err_msg)+1);
		node->next->next = NULL;

		strcpy(node->next->m, err_msg);
	}

	return 0;

}

/* --------------------------------------------------------------------- */

int reset_ts_struct()


{
	if (this_ts.ts_started)
		free_err_node(&(this_ts.msg));

	memset(&this_ts, 0, sizeof(this_ts));
}

/* --------------------------------------------------------------------- */

int free_err_node(n)
struct node *n;

{

	if (n->next == NULL) {
		if (n->m) {
		    free(n->m);
		    n->m = NULL;
		}
		return;
	} 

	free_err_node(n->next);
		

}


/* --------------------------------------------------------------------- */

int print_ts_error()

{
	struct node *n;
	char time_span_time[256];
	char time_span_scnl[256];

	if (this_ts.ts_started == 0)
		return 0;

	sprintf (time_span_time, "Time span %d,%d,%d:%d:%d.%d-%d,%d,%d:%d:%d.%d",
			this_ts.start.year,
				this_ts.start.day,
				this_ts.start.hour,
				this_ts.start.minute,
				this_ts.start.second,
				this_ts.start.fracsec,
                                this_ts.end.year,
                                this_ts.end.day, 
                                this_ts.end.hour, 
                                this_ts.end.minute, 
                                this_ts.end.second,
                                this_ts.end.fracsec);
	sprintf(time_span_scnl, "for stn/chn/net/loc: %s, %s, %s, %s", 
				this_ts.stn,
				this_ts.chn,
				this_ts.net,
				this_ts.loc);


	n = &this_ts.msg;
	
	do 
	{
		fprintf(stderr, "%s %s %s\n", n->m, time_span_time, time_span_scnl);

		n = n->next;

	} while (n != NULL);

}

/* --------------------------------------------------------------------- */


