/*===========================================================================*/
/* SEED reader     |               verseed                 |    main program */
/*                 |              Release 1.0              |                 */
/*===========================================================================*/
/*
	Name:	verseed
	Purpose:read Standard Exchange of Earthquake Data (SEED) format volume
	Usage:	Current usage is:
			verseed inputfile -{1 | 2 | 3 | 4 }

	Input:	an input file or device on which the SEED data reside
			a function switch, from the list given above
	Output:	for function 'a':  the abbreviation dictionary is written to
		the standard output.
		for function 't':  start/stop times for all events on the volume
	Warnings:several warnings are given by subprocedures.  Typical response
		to a warning is that a message is sent to the standard error
		device and execution continues.
	Errors:	various errors are trapped by subprocedures.  Typical response
		to an error is that a message is sent to the standard error
			device and execution is terminated.
	Called by:user
	Calls to:Each logical record type has its own processor, as listed below.
		Each processor includes several parsing routines which parse
		the various blockette types; the parsing routines are described
		more fully in the leading comments of the process_xxxx source
		files.  The processing routines are:
	Algorithm:Open the input file for reading; if the input file cannot be 
		opened, issue an error message and quit.  Process the input
		file one logical record at a time; each record type has an
		initial type flag which is used by this procedure to determine
		which type of processing to apply.  The information contained
		in each header type (except volume) is concatenated into a 
		volume-inclusive table for each different type:  six
		abbreviation dictionaries containing data format, comment
		description, cited source, generic abbreviation, units
		abbreviation, and beam configuration dictionaries (see Halbert
		et al.); a station header table containing complete information
		for each channel of each station; and a volume time span index.

	Language:C, hopefully to ANSI standard for portability.
	Author:	Allen Nance, adapted from rdseed written by Dennis O'Neill and Allen Nance
	Revisions:08/01/92  Alllen Nance  Initial preliminary release 1.0
	04/29/99  Stephane Zuzlewski - Integration of polynomial response


*/

#define MAIN
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "rdseed.h"				/* rdseed headers */

#include <termios.h>

#include <errno.h>
#include "patchlevel.h"

/* --------------------------------------------------------------------- */
#define DATE_IS_GARBAGE(d) ((d.year < 1960) || (d.year > 2010) || \
        (d.day == 0) || (d.day > 366))



int relax_flag = 0;
int alt_resp   = 0;

FILE *alt_resp_file;

extern int errno;

extern char *getenv();			/* define getenv system call */
void	usage ();				/* fcn to show usage */


/* function control flags, defined here for local subprocedure access */
static int response_flag = TRUE;		/* validate response flag */
static int times_flag = TRUE;			/* validate data time spans flag */
static int stage_warn_count;			/* count of response warning messages */
static char inputfilename[100];			/* name of input file */

static int Volume_start;
static int prev_Seq_Num = 0;

/* ---------------------------------------------------------------- */

main (argc, argv)
int argc;
char *argv[];
{
	int i;					/* counter */
	int j;					/* counter */
	int eof_count;				/* count of EOF marks found */
	int opt_count;				/* count of options found */
	struct input_data_hdr *input_data_hdr;	/* fixed data header */
	char *input_data_ptr;			/* fixed data header */
	struct optstruct *options;
	int data_present;

	int alt_resp_processing;
	int micro_seed_processing;
	int lcl_num_bytes_read, bytes_read;

	char *alt_resp_fname;

	int default_lrecl, data_lrecl;
	long pos;

	fprintf (stderr, "<< IRIS SEED Verifier, Release %s >>\n", PATCHLEVEL );

/*                 +=======================================+                 */
/*=================|    Process command line, set flags    |=================*/
/*                 +=======================================+                 */

	/* options:
		f;
		1-4:  	error report level
	*/

	default_lrecl = LRECL = 4096;	/* start at 256 until volume header has been read */

	
	input_data_hdr = (struct input_data_hdr *) input.data;
	outputfile = stdout;
	volume_number = 1;
	seis_buffer_length = MAX_DATA_LENGTH;
	strcpy(inputfilename,"/dev/nrst0");
	opt_count = 0;
	number_of_blk74 = number_of_time_series = 0;
	response_error = time_span_error = data_record_error = 0;
	response_overlap_error = 0;
	inputfile = stdin;
	adj_tolerance = 1.0;

	sniff_flag = FALSE;
	data_present = FALSE;
	alt_resp_processing = 0;
	micro_seed_processing = 0;

	times_flag = FALSE;
	stage_warn_count = 0;

	Volume_start = 0;

	/* read in alias strings */
	get_environment();

	if (argc > 1)	/* args present besides program name ? */
	{
		for (i = 1; i < argc; i++)
		{
			if (strcmp(argv[i], "-1") == 0)
			{
				sniff_flag = 1;
				continue;
			}
			if (strcmp(argv[i], "-2") == 0)
			{
                		sniff_flag = 2;
				continue;
			}
			if (strcmp(argv[i], "-3") == 0)
			{
                		sniff_flag = 3;
				continue;
			}

			if (strcmp(argv[i], "-4") == 0)
			{
                		sniff_flag = 4;
				continue;
			}	

			if (strcmp(argv[i], "-r") == 0)
			{
				relax_flag = TRUE;
				continue;
			}

			if (strcmp(argv[i], "-a") == 0)
			{
				alt_resp = TRUE;
				alt_resp_fname = argv[i+1];
				/* need to incr counter past the file name */
				i++;
				continue;
			}
			
			if (strcmp(argv[i], "-m") == 0)
			{
				micro_seed_processing = 1;
				continue;
			}

			strcpy (inputfilename, argv[i]);
            
			fprintf (stderr,"\tTaking input from %s\n", inputfilename);
            		/* determine type of file character or block */
            		input_file_type = check_block_mode(inputfilename);

            		if ((inputfile = fopen(inputfilename, "r")) == NULL)
            		{
            			fprintf (stderr, "\tERROR (verseed):  ");
            			fprintf (stderr,
            				"Input file %s unavailable for reading.\n",
            				inputfilename);
            			fprintf (stderr, "\tExecution terminated.\n");
            			exit (2);
            		}
		
		}

	}
	else
		if (stdin_empty())
		{
			usage();
			exit(-1);
		}

	if (inputfile == stdin)
		if (stdin_empty())
                {
                        usage();
                        exit(-1);
                }

/*                 +=======================================+                 */
/*=================|             Initialization            |=================*/
/*                 +=======================================+                 */

	if (alt_resp)
	{
		alt_resp_processing = TRUE;

		alt_resp_file = fopen(alt_resp_fname, "r");

		if (alt_resp_file == NULL)
		{

			fprintf (stderr, "\tERROR (verseed):  ");
			fprintf (stderr,
				"Error: Alt response file %s unavailable for reading.\n", alt_resp_fname);

			perror("verseed");

			alt_resp = FALSE;

			alt_resp_file = NULL;

			exit(-1);

		}

	}
	else
	{
		/* user must specify alt_response_file for micro seed */
		if (micro_seed_processing)
		{
			usage();
			exit(-1);
		}
	}


	init_data_hdr ();
	start_record = 0;
	offset = 0;
	more_data = TRUE; eof_count = 0;
	found_lrecl_flag = at_volume = FALSE; volume_count = 0;
	sample_rate_accum = 0.0;
	sample_rate_count = 0;

/*            +=======================================+                 */
/*============| Read and process file until EOF found |=================*/
/*            +=======================================+                 */

	while (more_data == TRUE)
	{
		if (offset == 0) /* read a physical record */
		{

	/* read a physical record and point to its beginning */
	/**************************************************************/
	/* WARNING: the "read" function is not defined in ANSI C.     */
	/* I would have preferred to use "fread", but it didn't work. */
	/*							      */
	/* Seems to be a problem reading from tape drive using fread  */
	/**************************************************************/

	/* use of "fileno" macro allows use of read w/ "fopen" and so on */

		  
		        num_bytes_read = 
				read_full_record((alt_resp ? 
						    alt_resp_file : 
						    inputfile), 
						  precord, 
						  PRECL);

			/* if end of file */
			if (num_bytes_read == 0)
			{
				if (alt_resp)
				{

					fclose(alt_resp_file);

					alt_resp = FALSE;

					/* trigger read in skip_to_D */

					offset = PRECL; 

					num_bytes_read = skip_to_D(inputfile);
				
				}
				else
				{
					if (volume_count) 
						break;
					else
					{
						fprintf(stderr,"!!! ERROR - Volume Not Found !!!\n");
						fclose(inputfile);
						exit(2);
					}
				}
			}		/* if bytes read == 0 */
			
			precord_ptr = (char *)precord;

			/* point to beginning of this logical record */

                	lrecord_ptr = precord_ptr;

		}	/* if offset == 0, time for read */


alt_label:

		input_data_hdr = (struct input_data_hdr *) (lrecord_ptr + 8);


		/* extract a logical record from the physical record */
		read_logical_record (lrecord_ptr);
#if 0

		if (prev_Seq_Num != 0)
		{
                	if ((prev_Seq_Num + 1) != input.recordnumber)
                	{
                        	fprintf(stderr, "WARNING : Logical record number is out of sequence. Check record number %d.", input.recordnumber);

                        	fprintf(stderr, "Expected %d.\n", prev_Seq_Num + 1);
                	}

        	}

		prev_Seq_Num = input.recordnumber;

#endif

		/* process logical record according to its type */
		/* "read_blockette" handles spans across logical or physical records */
		if (found_lrecl_flag || input.type=='V')
		switch (input.type)
		{
			case 'D':
			case 'R':
			case 'Q':
			case 'M':

				if (!at_volume) break;
				
				if (alt_resp)
				{
					alt_resp = FALSE;

					fclose(alt_resp_file);

					/* trigger read in skip_to_D */
					offset = PRECL;

					memset(precord, 0, sizeof(precord));


					num_bytes_read = skip_to_D(inputfile);

					goto alt_label;


				}

				if (micro_seed_processing)
				{
					LRECL = 64;
					times_data(0);
					break;
				}

				/* do swap if needed before scan_for_LrecL() */
				input_data_ptr = lrecord_ptr+8;
				input_data_hdr = (struct input_data_hdr *)(input_data_ptr);

				if (DATE_IS_GARBAGE(input_data_hdr->time))
				{
					swap_fsdh(&input_data_hdr, &input_data_ptr);
					set_fsdh_swapped_flag(1);
				}
				else
					set_fsdh_swapped_flag(0);

				for (data_lrecl = scan_for_LrecL(lrecord_ptr + 8),i=0;
					i < LRECL/data_lrecl; i++)
				{
					/* Move to next data block within logical block */
					if(i){
						if(lrecord_ptr >= &precord[PRECL]){
							fprintf(stderr, "Trying to access data beyond end of physical block buffer\n");
							exit(1);
						}
						read_logical_record (lrecord_ptr);
					}

					if (sniff_flag == 4)
					{
						fprintf(stderr, "Data Record:%6.6s\n", lrecord_ptr);
					}	

					/* weed out blank recs, which is data_rec_size < logrec
					 * could exists in order to pad out Log record
					 */
					if (!all_blank(lrecord_ptr + 8, 10))
					{

						times_data (0);
					}

					lrecord_ptr += data_lrecl;
				}	

				/* we only want to run the below if we are at the 
					beginning of a data header */

				data_present = TRUE;

               			/* set up offset for next time around */
                		offset += LRECL;

                		if (offset >= num_bytes_read)
                        		offset = 0;

				continue;

			case 'T':
				if (!at_volume) break;

				if (sniff_flag == 4)
				{
					fprintf(stderr, "Time Span Record\n");
				}
				process_timeh ();

				break;
			case 'S':

				if (!at_volume) break;
				if (sniff_flag == 4)
				{
					fprintf(stderr, "Station header record\n");
				}
				process_stnh ();
				break;
			case 'A':
				if (!at_volume) break;
				if (sniff_flag == 4)
				{
					fprintf(stderr, "Abbreviation dictionary record\n");
				}
				process_abrvd ();
				break;
			case 'V':
				reset_ts_struct(); 

				if (sniff_flag == 4)
				{
					fprintf(stderr, "Volume header record\n");
				}

				if (volume_count)
				{
					check_responses();
					if (data_present) times_data (1);
					if (response_error || time_span_error || data_record_error )
					{
						fprintf (stderr, "ERRORS - ");
						if (response_error) fprintf(stderr, "\t%d response blockette errors\n", response_error);
						if (time_span_error) fprintf(stderr, "\t%d time span blockette errors\n", time_span_error);
						if (data_record_error) fprintf(stderr, "\t%d data record errors\n", data_record_error);
						fprintf (stderr, "\n");
					}

					if (number_of_blk74 != number_of_time_series)
					{
						fprintf (stderr, "ERRORS - ");

fprintf (stderr, "\t# Time Spans (Blk 74s) = %d, not equal to # Time Series (data)= %d\n", number_of_blk74, number_of_time_series);
					}

					if (number_of_time_series == 0)
					{
						fprintf (stderr, "WARNING - No Data Records found in Volume\n");
					}
					free_all();
				}

				start_record = number_of_blk74 = number_of_time_series = 0;
				response_error = time_span_error = data_record_error = 0;
				response_overlap_error = 0;
				volume_count++; at_volume = TRUE; data_present = FALSE;

				fprintf(stderr, "\n==================\n");
				fprintf(stderr, "Processing Volume %d    \n", volume_count);

                               /* minus the 32k read in above */
				/*Volume_start = ftell(alt_resp?alt_resp_file:inputfile) - 32768;*/
				Volume_start = ftell(alt_resp?alt_resp_file:inputfile) - num_bytes_read;

				process_volh ();

				/* LRECL was read in from blk10 here */
				default_lrecl = LRECL;

				break;
			case ' ':
				if (!at_volume) break;
				if (sniff_flag == 4)
				{
					fprintf(stderr, "Record number %ld was blank\n",input.recordnumber);
					break;
				}
				process_blank ();
				break;
			default:
				if (input.recordnumber == 0) {
					fprintf (stderr, "ERROR (verseed):  ");
					fprintf (stderr, "Record number %ld was of unknown type %c.\n", 
						input.recordnumber, input.type);
                                        fprintf (stderr, "\tExecution terminated.\n");
                                        exit (2);
                                }
				fprintf (stderr, "WARNING (verseed):  ");
				fprintf (stderr, "Record number %ld was of unknown type %c.\n", 
					input.recordnumber, input.type);
				fprintf (stderr, "\tExecution continuing.\n");
		}


		lrecord_ptr = lrecord_ptr + LRECL;

		/* set up offset for next time around */
		offset += LRECL;
		if (offset >= num_bytes_read)
                        offset = 0;

	}

	if (data_present) 
	{
		if (seekable(inputfile) && (!alt_resp_processing))
			scan_74s(inputfile);

		times_data (1);

	}

	print_ts_error();

	if (!alt_resp_processing)
		check_11(inputfile);

	if (!alt_resp_processing && data_present)
		check_12(inputfile);

	check_responses(relax_flag);

/*                 +=======================================+                 */
/*=================|   Report contents of volume headers   |=================*/
/*                 +=======================================+                 */

/*                 +=======================================+                 */
/*=================|                clean up               |=================*/
/*                 +=======================================+                 */

	if (outputfile != stdout) fclose (outputfile);
	fclose (inputfile);

	if (stage_warn_count)
	{
		fprintf (stderr, "WARNING - ");
		fprintf(stderr, "\t%d response stage ommissions found\n", stage_warn_count);
	}

	if (response_error || response_overlap_error || time_span_error || data_record_error )
	{
		fprintf (stderr, "ERRORS - ");
		if (response_error) fprintf(stderr, "\t%d response blockette errors\n", response_error);
		if (response_overlap_error) fprintf(stderr, "\t%d response overlap blockette errors\n", response_overlap_error);
		if (time_span_error) fprintf(stderr, "\t%d time span blockette errors\n", time_span_error);
		if (data_record_error) fprintf(stderr, "\t%d data record errors\n", data_record_error);
		fprintf (stderr, "\n");
	}

	if (number_of_blk74 != number_of_time_series)
	{
		fprintf (stderr, "Warning - ");

		fprintf (stderr, "\t# Time Spans = %d, not equal to # Time Series = %d\n", number_of_blk74, number_of_time_series);

		fprintf(stderr, "This could be due to time series that crosses a midnight boundary\n");

	}

	if (number_of_time_series == 0)
	{
		fprintf (stderr, "WARNING - No Data Records found in Volume\n");
	}

	fprintf (stderr, "verseed completed.\n");
	if (response_error || response_overlap_error || time_span_error || data_record_error ) exit(1);
	else exit(0);
}

/*===========================================================================*/
/* SEED reader     |                 usage                 |    subprocedure */
/*===========================================================================*/
/* show usage of program */

void usage ()
{
	fprintf(stderr, "Usage:  verseed inputfile -[1..4, r, a <file_name>, m]\n");
	fprintf(stderr, "\twhere inputfile = name of input file or device;\n");
	fprintf(stderr, "\t1..4  = error reporting level;\n");
	fprintf(stderr, "\tr = relaxed error reporting on special errors\n");
	fprintf(stderr, "\ta = alternate response file processing\n");
	fprintf(stderr, "\tm = microseed processing (assumes the \"a\" option is active)\n");
	fprintf(stderr, "\tIf no file name is present, verseed uses standard in.\n\n"); 
}
/*=======================================================================*/
/* SEED reader     |               skip_to_D           |    main program */
/*=======================================================================*/
int skip_to_D(fptr)
FILE *fptr;

{

        int num_bytes_read;
	long pos;
 
	do
	{ 
                if (offset + LRECL > PRECL)
                {
		  /* check HERE */
		        num_bytes_read = read_full_record((alt_resp ? alt_resp_file : inputfile), precord, PRECL);
/*                         num_bytes_read = read(fileno(fptr), precord, PRECL); */
/* 			fprintf(stderr,"read %d bytes, asked for %d bytes\n", num_bytes_read, PRECL); */
			/* fgetpos(fptr, &pos);
			num_bytes_read = PRECL * fread(precord, PRECL, 1, fptr); */

/*                         if (num_bytes_read < 0) */
/*                         { */
/*                                 fprintf(stderr, "ERROR: skip_to_D(): unable to read the inputfile scanning for data records\n"); */

/*                                 perror("verseed"); */
 
/*                                 return 0; */
 
/*                         } */
        
                        /* didn't find any */
                        if (num_bytes_read == 0)
                                return 0;
 
                        offset = 0;
                        lrecord_ptr = precord;
                }
 
                if (lrecord_ptr[6] == 'D')
                        break;
                if (lrecord_ptr[6] == 'R')
                        break;
                if (lrecord_ptr[6] == 'Q')
                        break;
                if (lrecord_ptr[6] == 'M')
                        break;
        
                lrecord_ptr += LRECL;
                offset += LRECL;

       
        } while (1);
 
        return num_bytes_read;
 
}

/*=======================================================================*/
/* SEED reader     |               strupr              |    main program */
/*=======================================================================*/

/* convert string to upper case */
strupr(string)
char *string;
{
	while (*string != '\0')
		{
		if (islower(*string)) *string = toupper(*string);
		string++;
		}
}

/*=======================================================================*/
/* SEED reader     |           get_environment         |    main program */
/*=======================================================================*/
/* get rdseed environment variables */
get_environment()
{
	FILE *fp;
	char buffer[200], *p, *p2;
	int i, j;

	p = getenv("SEEDALIAS");
	if (p != NULL)
	{
	}
}

/*=======================================================================*/
/* SEED reader     |         check_block_mode          |    main program */
/*=======================================================================*/
/* can file be read in block mode? */
check_block_mode(filename)
char *filename;
{
	struct stat buf;

	memset(&buf, 0, sizeof(buf));

	stat(filename, &buf);

	if (buf.st_mode & S_IFCHR) 
		return(0);
	else 
		return(-1);
}

/*=======================================================================*/
/* SEED reader     |           strip_blanks            |    main program */
/*=======================================================================*/
/* strip blanks from string */
strip_blanks(input, output)
char *input, *output;
{
	int j; char *p, buf[100];

	p = input; j = 0;
	while (*p != '\0')
	{
		if (*p != ' ') {buf[j] = *p; j++; }
		p += 1;
	}
	buf[j] = '\0';
	strcpy(output,buf);
}

/* ------------------------------------------------------------------- */
/* check array for non-blank chars */
int all_blank(array, len)
char array[];
int len;

{
	int i;

	for (i = 0; i < len; i++)
		if (!(isspace(array[i])))
			return 0;

	return 1;

}

/* ------------------------------------------------------------------- */
struct fsdh
{
        char   SequenceNumber[6] ;
        char   Data_header_indicator ;
        char   Reserved_bytes_A ;
        struct input_data_hdr hdr;   /* fixed section of header */
};
/* ------------------------------------------------------------------- */
#define BLANK_REC 2

int chk_data_lrecl(l, buffer)
int l;
char *buffer;

{

	struct fsdh *fsdh;
	int try;

	int result;

	try = 512;

	while (try < 4096)
	{
		if (result = check_out_fsdh((struct fsdh *)&buffer[try]))
			break;

		try += try;

	}	/* while <= 4096 */

	if (result == BLANK_REC)
		return l;	/* give up on this rec if blanks encountered */

	if (try != l)
	{
		char tmp_str[255];

		char stn[10];
		char chn[10];
		char loc[10];
		char net[10];

		fsdh = (struct fsdh *)&buffer[0];

		sprintf(tmp_str, "ERROR: Inconsistancy detected btw stn/chn/blk_1k data_recsize : %d, and what appears when scanning: %d", l, try);
 		
		sprintf(stn, "%5.5s", fsdh->hdr.station);
		sprintf(chn, "%3.3s", fsdh->hdr.channel);
		sprintf(loc, "%2.2s", fsdh->hdr.location);
		sprintf(net, "%2.2s", fsdh->hdr.network);

		trim(stn); trim(chn); trim(loc); trim(net);

		if (sniff_flag == 0)
		{
			log_tspan_error(stn, chn, 
					net, loc, 
					&(fsdh->hdr.time),	
					tmp_str);
		}
		else
		{
			fprintf(stderr, "%s\n", tmp_str);

			fprintf(stderr, "\tfor Stn/chn/loc/net: %s, %s, %s, %s\n",
					stn, chn,
                                        loc, net);

			fprintf(stderr, "\t\ttime: %d, %d, %d:%d:%d.%d\n",
					fsdh->hdr.time.year,
					fsdh->hdr.time.day,
					fsdh->hdr.time.hour,
					fsdh->hdr.time.minute,
					fsdh->hdr.time.second,
					fsdh->hdr.time.fracsec);
		}
	}

	return try;

}

/* ---------------------------------------------------------------------- */
int check_out_fsdh(f)
struct fsdh *f;

{
	int i;

	if (all_blank(&f->hdr, sizeof(f->hdr)))
		return BLANK_REC;

	/* check for start 00000D - all ascii */
	for (i = 0; i < sizeof(f->SequenceNumber); i++)
		if (!isdigit(f->SequenceNumber[i]))
			return 0;

	if ((f->Data_header_indicator != 'D') && 
		(f->Data_header_indicator != 'R') && 
		(f->Data_header_indicator != 'M') && 
		(f->Data_header_indicator != 'Q'))
		return 0;

	/* logrec could be padded with blank rec */
	if (DATE_IS_GARBAGE(f->hdr.time))
	{
		struct input_data_hdr *ptr = &f->hdr;

       		swap_fsdh(&ptr, &ptr);

	}

	/* check time */
	if (!((f->hdr.time.year > 1970 && f->hdr.time.year < 2010) 	&&
		(f->hdr.time.day >= 1 && f->hdr.time.day <= 366) 	&&
		(f->hdr.time.hour >= 0 && f->hdr.time.hour <= 59) 	&&
		(f->hdr.time.minute >= 0 && f->hdr.time.minute <= 59) 	&&
		(f->hdr.time.second >= 0 && f->hdr.time.second <= 59) 	&&
		(f->hdr.time.fracsec <= 9999)))
			return 0;


	for (i = 0; i < sizeof(f->hdr.station); i++)
		if (!isascii(f->hdr.station[i]))
			return 0;

	for (i = 0; i < sizeof(f->hdr.location); i++)
		if (!isascii(f->hdr.location[i]))
                        return 0;

	for (i = 0; i < sizeof(f->hdr.channel); i++) 
                if (!isascii(f->hdr.channel[i])) 
                        return 0; 
 
        for (i = 0; i < sizeof(f->hdr.network); i++) 
                if (!isascii(f->hdr.network[i])) 
                        return 0; 
	
	return 1;
	
}

/* ------------------------------------------------------------------- */

/* ------------------------------------------------------------------- */

/* ------------------------------------------------------------------- */

/* ------------------------------------------------------------------- */


int check_11(inputfile)
FILE *inputfile;


{
	int i, result;

	char buff[30];

	char stn[6];

	if (!seekable(inputfile))
		return 0;
 
	for (i = 0; i < type11.number_stations; i++)
	{
                result = lseek(fileno(inputfile), 
				((type11.station[i].sequence_number - 1) * LRECL) + Volume_start, 0);

                if (result < 0)
                {
                        fprintf(stderr, "ERROR - check_11, Unable to seek to data position.\n");
                        perror("check_11");
                        return 0;
                }

		/* read in the station name and compare */
		if ((result = read(fileno(inputfile), buff, sizeof(buff))) != sizeof(buff))
		{
			fprintf(stderr, "I/O error processing blockette 11!\n");

			fprintf(stderr, 
				"Unable to check station %s from b11\n", 
				type11.station[i].station_id == NULL ?
					"" : type11.station[i].station_id);

			fprintf(stderr, 
				"Giving up on checking blockette 11\n");

			perror("check11");

			return 0;

		}

		/* station letters start at position 15 and extend for 5 */
		strncpy(stn, &buff[15], 5);
		stn[5] = 0;

		trim(stn);

		if (strcmp(type11.station[i].station_id, stn) != 0)
		{
			fprintf(stderr, "Error, blockette 11 indexes are off.!\n");

			fprintf(stderr, 
				"Expected station %s, found %s\n",
				type11.station[i].station_id == NULL ?
                                   "" : type11.station[i].station_id,
				stn);
		}
			
	} 	/* for all stations */

	return 1;

}

/* ------------------------------------------------------------------- */
int check_12(inputfile)
FILE *inputfile;

{
        int i;
        int result;

	char buff[10];

        if (!seekable(inputfile))
                return 0;

        for (i = 0; i < type12.number_spans; i++)
        {

                result = lseek(fileno(inputfile),
                                ((type12.timespan[i].sequence_number - 1) * LRECL) + Volume_start, 0);

                if (result < 0)
                {
                        fprintf(stderr, "ERROR - check_12, Unable to seek to data position.\n");
                        perror("check_12");
                        return 0;
                }

                if (read(fileno(inputfile), buff, 8) != 8)
		{
			fprintf(stderr, 
				"I/O error processing blockette 12!\n");

				fprintf(stderr, 
					"check_12() : Unable to check blockette 12.\n"); 

                        fprintf(stderr, 
				"Giving up on checking blockette 12.\n");

                        perror("check12");

                        return 0;

		}

		/* check to see that the 'T' is there */
		if (buff[6] != 'T')
		{
			fprintf(stderr, "Error, bad blockette 12 detected. Improper index.\n");

			fprintf(stderr, "Unable to locate the timespan header section using the blockette 12 index.\n");

			return 0;

		}

	}

}

/* ------------------------------------------------------------------- */


check_responses(relax)
int relax;

{
	struct type43 *type43_ptr = type43_head;
	struct type52 *type52;
	struct type50 *p_station;		/* pointer save locations */
	struct response *response;					/* looping vbl */
	short yes53, yes43;
	short yes62, yes42;

	char cmp_station[6];
	int type52_count, i;
	struct
	{
		char yes53;
		char yes54;
		char yes57;
		char yes58;	
		char yes62;
		char yes42;
		char yes43;
		char yes44;
		char yes47;
		char yes48;
	} check[20];

	while (type43_ptr)
	{
		qa_type43(type43_ptr);

		type43_ptr = type43_ptr->next;
	}

	cmp_station[0] = 0; type52_count = 0;

	/* find the current station information in the tables */
	for (p_station = type50_head;
			p_station != NULL;
			p_station = p_station->next)
	{		

		if ((strcmp(cmp_station, p_station->station) != 0) && (p_station != type50_head))
		{
			if (type52_count == 0)
			{
				fprintf(stderr, "ERROR - ");
				fprintf(stderr, "\tNo blockette 52's found for station %s\n", cmp_station);
			}
			type52_count = 0;
		}

		strcpy(cmp_station, p_station->station);

	/* find the current channel within the current station */
		for (type52 = p_station->type52_head;
				type52 != NULL;
				type52 = type52->next)
		{
			type52_count++;

		       	/* skip if a LOG record */
        		if (strcasecmp(type52->channel, "LOG") == 0)
                		continue;

			if (type52->response_head != NULL)
			{

				for (i=0; i < 20; i++) {
					check[i].yes53 = FALSE;
					check[i].yes54 = FALSE;
					check[i].yes57 = FALSE;
					check[i].yes58 = FALSE;
					check[i].yes62 = FALSE;
					check[i].yes42 = FALSE;
					check[i].yes43 = FALSE;
					check[i].yes44 = FALSE;
					check[i].yes47 = FALSE;
					check[i].yes48 = FALSE;
				} 

				yes43 = FALSE;
				yes53 = FALSE;
				yes62 = FALSE;
				yes42 = FALSE;


	for (response = type52->response_head;
			response != NULL; response = response->next)
	{
		if (response->type == 'P')
		{
			char error_byte = 0;

			if (response->ptr.type53->stage < 20)
				check[response->ptr.type53->stage].yes53 = TRUE;
			yes53 = TRUE;

			error_byte = qa_type53(response->ptr.type53);

			process_error(error_byte,
					p_station,
					type52);

		}
		else if (response->type == 'O')
		{
			if (response->ptr.type62->stage
< 20)
				check[response->ptr.type62->stage].yes62 = TRUE;
			yes62 = TRUE;

		}
		else 
		if (response->type == 'C')
		{
			if (response->ptr.type54->stage < 20)
				check[response->ptr.type54->stage].yes54 = TRUE;
		}
		else 
		if (response->type == 'D')
		{
			if (response->ptr.type57->stage < 20)
				check[response->ptr.type57->stage].yes57 = TRUE;
		}
		else 
		if (response->type == 'S')
		{
			if (response->ptr.type58->stage < 20)
				check[response->ptr.type58->stage].yes58 = TRUE;
		}
		else if (response->type == 'R')
		{
			struct type60 *type60;		/* looping vbl */
			int i, j;

			for (type60 = response->ptr.type60; type60 != NULL; type60 = type60->next)
			{
				for( i = 0; i < type60->number_stages; i++ )
				{
					for( j = 0; j < type60->stage[i].number_responses; j++)
					{
						if (find_type43(NULL,type60->stage[i].response[j].reference))
						{
							if (type60->stage[i].value < 20)
								check[type60->stage[i].value].yes43 = TRUE;
								yes43 = TRUE;
						}
						else
						if (find_type42(NULL,type60->stage[i].response[j].reference))                    
						{
							if (type60->stage[i].value < 20)
								check[type60->stage[i].value].yes42 = TRUE;
							yes42 = TRUE;
						}
						else
						if (find_type44(NULL,type60->stage[i].response[j].reference))
						{
							if (type60->stage[i].value < 20)
								check[type60->stage[i].value].yes44 = TRUE;
						}
						else
						if (find_type47(NULL,type60->stage[i].response[j].reference))
						{
							if (type60->stage[i].value < 20)
								check[type60->stage[i].value].yes47 = TRUE;
						}
						else
						if (find_type48(NULL,type60->stage[i].response[j].reference))
						{
							if (type60->stage[i].value < 20)
								check[type60->stage[i].value].yes48 = TRUE;
						}
					}
				}
			}
		}
	}


				for (i=1;i<20;i++)
				{
					if (check[i].yes53 && !check[i].yes58)
					{
						stage_warn_count++;
						if (sniff_flag >= 3)
						{
						fprintf (stderr, "WARNING - ");
						fprintf(stderr, "\tGain blockette 58 for 53 not found in stage %d %s,%s,%s\n", i, p_station->network_code,p_station->station, type52->channel);
						}
					}
					if (check[i].yes43 && !check[i].yes48)
					{
						stage_warn_count++;
						if (sniff_flag >= 3)
						{
						fprintf (stderr, "WARNING - ");
						fprintf(stderr, "\tGain blockette 48 for 43 not found in stage %d %s,%s,%s\n", i, p_station->network_code,p_station->station, type52->channel);
						}
					}
                                        if (check[i].yes62 && check[i].yes58)
                                        {
                                                stage_warn_count++;
                                                if (sniff_flag >= 3)
                                                {
                                                fprintf (stderr, "WARNING - ");
                                                fprintf(stderr, "\tGain blockette 58 for 62 found in stage %d %s,%s,%s\n", i, p_station->network_code,p_station->station, type52->channel);                                                }
                                        }
                                        if (check[i].yes42 && check[i].yes48)
                                        {
                                                stage_warn_count++;
                                                if (sniff_flag >= 3)
                                                {
                                                	fprintf (stderr, "WARNING - ");
                                                	fprintf(stderr, "\tGain blockette 48 for 42 found in stage %d %s,%s,%s\n", i, p_station->network_code,p_station->station, type52->channel);                                             
						}

					}
					if (check[i].yes54 && !check[i].yes58)
					{
						stage_warn_count++;
						if (sniff_flag >= 3)
						{
						fprintf (stderr, "WARNING - ");
						fprintf(stderr, "\tGain blockette 58 for 54 not found in stage %d %s,%s,%s\n", i, p_station->network_code,p_station->station, type52->channel);
						}
					}

					if (check[i].yes44 && !check[i].yes48)
					{
						stage_warn_count++;
						if (sniff_flag >= 3)
						{
						fprintf (stderr, "WARNING - ");
						fprintf(stderr, "\tGain blockette 48 for 44 not found in stage %d %s,%s,%s\n", i, p_station->network_code,p_station->station, type52->channel);
						}
					}

					if (check[i].yes54 && !check[i].yes57)
					{
						stage_warn_count++;
						if (sniff_flag >= 3)
						{
						fprintf (stderr, "WARNING - ");
						fprintf(stderr, "\tDecimation blockette 57 for 54 not found in stage %d %s,%s,%s\n", i, p_station->network_code,p_station->station, type52->channel);
						}
					}

					if (check[i].yes44 && !check[i].yes47)
					{
						stage_warn_count++;
						if (sniff_flag >= 3)
						{
						fprintf (stderr, "WARNING - ");
						fprintf(stderr, "\tDecimation blockette 47 for 44 not found in stage %d %s,%s,%s\n", i, p_station->network_code,p_station->station, type52->channel);
						}
					}
					
				}

				/*if ((!yes53 && !check[0].yes58) && 
				    (!yes43 && !check[0].yes48) &&
				    (!(yes62 || yes42)))
				{
					if (check[i].yes58)
						fprintf (stderr, "ERROR - ");

					fprintf(stderr, "\tMinimum response blockettes 43/53 and 48/58 stage 0 not found for %s,%s,%s\n", p_station->network_code,p_station->station, type52->channel);
					response_error++;
				}
				else if ((!yes53 && !yes43) && 
					 (check[0].yes58 || check[0].yes48) &&
					  (!yes62 && !yes42))
				{
					fprintf(stderr, "ERROR - ");
					fprintf(stderr, "\tMinimum response blockette 43/53 not found for %s,%s,%s\n", p_station->network_code,p_station->station, type52->channel);
					response_error++;
				}
				else if ((yes53 || yes43) && 
					 (!check[0].yes58 && !check[0].yes48) &&
					 (!yes62 && !yes42))
				{
					fprintf (stderr, "ERROR - ");
					fprintf(stderr, "\tMinimum response blockette 48/58 stage 0 not found for %s,%s,%s\n", p_station->network_code,p_station->station, type52->channel);
					response_error++;
				}
				else */if ((check[0].yes58 || check[0].yes48) && 
					 (yes62 || yes42))
                                {
                                        fprintf (stderr, "ERROR - ");

					fprintf(stderr, "\tGain blockette 48/58 stage 0 found for %s,%s,%s\n", p_station->network_code,p_station->station, type52->channel);

                                        response_error++;
                                }


			} else
			{
				fprintf (stderr, "Warning - ");
				fprintf(stderr, "\tNo response blockettes found for %s,%s,%s\n", p_station->network_code,p_station->station, type52->channel);
			}
		}
	}

	if (type50_head != NULL)
	{
		if (type52_count == 0)
		{
			fprintf(stderr, "ERROR - ");
			fprintf(stderr, "\tNo blockette 52's found for station %s\n", cmp_station);
		}
	}

	if (!relax)
		check_gains(type50_head);

	check_dimensions(type50_head);

	check_decimations(type50_head);

}

/* --------------------------------------------------------------------- */
int stdin_empty()

{
	
	int ret;
	struct stat sbuf;

	ret = fstat(0, &sbuf);   
 
        if (ret < 0)
        { 
                perror("stdin_empty"); 
                return FALSE;   /* assume not empty */ 
  
        } 
  
        if (sbuf.st_mode == S_IFIFO)  
                return FALSE;   
        else
        {
                if (sbuf.st_size == 0)
                {
                        return TRUE;
                }

	}

	return FALSE;  /* not empty */

}

/* ------------------------------------------------------------------------- */
int seekable(file_ptr)
FILE * file_ptr;

{
	int err;

	struct stat st;

	if(file_ptr == stdin) 
		return 0;

	err = fstat(fileno(file_ptr), &st);

	if (err == -1)
	{
		perror("seekable");
		return FALSE;
	}

	return S_ISREG(st.st_mode);

}

/* ------------------------------------------------------------------------ */
int check_blk_74s(fptr)
FILE *fptr;

{
	int err;

	struct type74 *t_74 = type74_head;

	while (t_74)
	{
		if (!t_74->start_index_chk)
			fprintf(stderr, "ERROR - missing time span's starting data block!\n\tStation: %s, channel: %s, start time: %s, end time:%s\n", 
					t_74->station, 
					t_74->channel, 
					t_74->starttime, 
					t_74->endtime);

		if (!t_74->end_index_chk)
                        fprintf(stderr, "ERROR - missing time span's ending data block!\n\tStation: %s, channel: %s, start time: %s, end time:%s\n", 
                                        t_74->station,
                                        t_74->channel, 
                                        t_74->starttime, 
                                        t_74->endtime);

		t_74 = t_74->next;

	}	/* while more t 74s */

	return 1;
}

/* ------------------------------------------------------------------------------ */
int scan_74s(fptr)
FILE *fptr;


{
        int err = 0;

	char station[10];
	char channel[10];
	char network[50];
	
	struct input_data_hdr buff;
	off_t result;
	int num_items;

        struct type74 *t_74 = type74_head;


        while (t_74)
        {
		result = lseek(fileno(inputfile), Volume_start +
				(((t_74->start_index-1)*LRECL)) + 8, 0);

               	if (result < 0) 
                {
                        fprintf(stderr, "ERROR - scan_74s, Unable to seek to data position.\n");
                        perror("scan_74s"); 
                        return 0; 
                } 

		num_items = read(fileno(inputfile), &buff, sizeof(buff));

		if (num_items != sizeof(buff))
		{
			fprintf(stderr, "ERROR - scan_74s - Unable to read the data! For block 74: network %s: station %s: channel %s\n", 
				t_74->network_code,
				t_74->station,
				t_74->channel);

			perror("scan_74s");

			return 0;
		}

	
		sprintf(station, "%5.5s", buff.station);
		sprintf(channel, "%3.3s", buff.channel); 
		sprintf(network, "%2.2s", buff.network); 

		trim(station);
		trim(network);
		trim(channel);

		err = (strcmp(t_74->station, station) != 0) ||
			(strcmp(t_74->channel, channel) != 0) ||
			(strcmp(t_74->network_code, network) != 0);

		if (err)
		{
			fprintf(stderr, "Warning - Found inconsistency between blockette 74' network, station, channel and the corresponding data record!\n");

			fprintf(stderr, "\texpected (B74) %s, %s, %s\n\t\tfound (Data record) %s, %s, %s\n",
						t_74->network_code, 
						t_74->station, 
						t_74->channel,
						network,
						station,
						channel);

		}

		/* check out the ending index value */

                result = lseek(fileno(inputfile), Volume_start +
                        	((t_74->end_index- 1)*LRECL) + 8, 0);

		if (result < 0)
		{
			fprintf(stderr, "ERROR - scan_74s, Unable to seek to data position.\n");
			perror("scan_74s");
			return 0;
		}

 
                num_items = read(fileno(inputfile), &buff, sizeof(buff));
 
                if (num_items != sizeof(buff))
                {
                        fprintf(stderr, "ERROR - scan_74s - Unable to read the data! For block 74: network %s: station %s: channel %s\n", 
                                t_74->network_code, 
                                t_74->station,    
                                t_74->channel);
 

                        perror("scan_74s");
 
                        return 0;
                }

                sprintf(station, "%5.5s", buff.station);
                sprintf(channel, "%3.3s", buff.channel);
                sprintf(network, "%2.2s", buff.network);

                trim(station);
                trim(network);
                trim(channel);
	
		err = (strcmp(t_74->station, station) != 0) ||
                	(strcmp(t_74->channel, channel) != 0) ||
                        (strcmp(t_74->network_code, network) != 0); 
 
		if (err)
		{
			fprintf(stderr, "Warning - Found inconsistency between blockette 74' network, station, channel and the corresponding data record!\n");

			fprintf(stderr, "\texpected (B74) %s, %s, %s\n\t\tfound (Data record) %s, %s, %s\n",
						t_74->network_code, 
						t_74->station, 
						t_74->channel,
						network,
						station,
						channel);

		}

		t_74 = t_74->next;
	}

	/* we will get here if in btw time span start/stop recs */
        return 1;
}

/* ------------------------------------------------------------------------------ */
int trim(s)
char s[];

{
	char *p;

	int l;

	l = strlen(s);

	while (!isalnum(s[l]))
	{
		s[l] = '\0';
		l--;

		if (l < 0)
			return;

	}
	
}

/* ------------------------------------------------------------------------ */
int scan_for_LrecL(data_header)
struct input_data_hdr *data_header;

{

	char tmp_str[255];

	int blk_1000_lrecl, default_lrecl, stn_chn_lrecl;

	char station[30];
        char channel[30];
        char network_code[10];
	char loc[5];
 
	default_lrecl = LRECL;

	strncpy(station, data_header->station, sizeof(data_header->station));
	station[sizeof(data_header->station)] = 0;
	trim(station);

	strncpy(channel, data_header->channel, sizeof(data_header->channel)); 
	channel[sizeof(data_header->channel)] = 0;
	trim(channel);

	strncpy(network_code, data_header->network, sizeof(data_header->network)); 
        network_code[sizeof(data_header->network)] = 0; 
        trim(network_code); 

        strncpy(loc, data_header->location, sizeof(data_header->location));
        loc[2] = 0;
 
        trim(loc);
 
	stn_chn_lrecl = get_stn_chn_Lrecl(station,
                                  	  channel,
                                  	  network_code,
				  	  loc,
                                  	  &(data_header->time));
 
        if (stn_chn_lrecl == 0 || (stn_chn_lrecl % 256 != 0))
        {
		
		sprintf(tmp_str, 
			"ERROR: Bad log rec length parsed from chn record. Defaulting to %d", default_lrecl);

		log_tspan_error(station, 
				channel, 
				network_code, 
				loc, 
				&(data_header->time),
				tmp_str);

                stn_chn_lrecl = default_lrecl;
 
        }
        

	/* Check blockette 1000, which will override, if there */
	/* If not there, the stn_chn_lrecl is returned by the routine */

	blk_1000_lrecl = get_blk_1000_Lrecl(data_header, stn_chn_lrecl);

	if (blk_1000_lrecl == 0 || (blk_1000_lrecl % 256 != 0))
        {
		sprintf(tmp_str, "ERROR: Bad logical record length scanned from blk 1000. Defaulting to:%d", default_lrecl);

		log_tspan_error(station, 
				channel, 
				network_code, 
				loc, 
				&(data_header->time),
				tmp_str);

		if (sniff_flag > 0)
		{

                        fprintf(stderr, "Bad Logical record length from blk 1000, scanned:%d\n", blk_1000_lrecl);

                        fprintf(stderr, "for stn/chn/loc/net:%s/%s/%s/%s",
                                        station,
                                        channel,
                                        loc,
                                        network_code);

                        fprintf(stderr, "\tdata rec time:%d,%d,%d:%d:%d.%d\n",
                                        data_header->time.year,
                                        data_header->time.day,
                                        data_header->time.hour,
                                        data_header->time.minute,
                                        data_header->time.second,
                                        data_header->time.fracsec);
		}
 
                blk_1000_lrecl = default_lrecl;
 
        }

	if (stn_chn_lrecl != blk_1000_lrecl)
	{

		if (sniff_flag > 0)
		{

			fprintf(stderr, "WARNING: Blk 52 data record length:%d, is inconsistent with the blk 1000's:%d (default)\n",
					stn_chn_lrecl, blk_1000_lrecl);

                        fprintf(stderr, "\tfor stn/chn/loc/net:%s/%s/%s/%s",
                                        station,
                                        channel,
                                        loc,
                                        network_code);

                        fprintf(stderr, "\t\tdata rec time:%d,%d,%d:%d:%d.%d\n",
                                        data_header->time.year,
                                        data_header->time.day,
                                        data_header->time.hour,
                                        data_header->time.minute,
                                        data_header->time.second,
                                        data_header->time.fracsec);

                }

	}

	if (blk_1000_lrecl != LRECL)
	{
		blk_1000_lrecl = 
			chk_data_lrecl(blk_1000_lrecl, 
						((char *)data_header) - 8);
	}
	
	/* always use blk_1000_lrecl which holds the stn_chn_lrecl if unused */
	return blk_1000_lrecl;
}

/* ------------------------------------------------------------------------- */
/* ------------------------------------------------------------------------- */
struct type50 *get_station_rec();
struct type52 *get_channel_rec();

/* ------------------------------------------------------------------------- */
int get_stn_chn_Lrecl(s, c, n, l, t)
/* station, channel, net, time */
char *s, *c, *n, *l;
struct input_time *t;            

{
        struct time rec_time;
	
	/* the only difference is shorts to ints */
        rec_time.year 	= t->year;
	rec_time.day 	= t->day;
	rec_time.hour 	= t->hour;
	rec_time.minute = t->minute;
	rec_time.second = t->second;
	rec_time.fracsec= t->fracsec;

	/* check if byteswapping will be needed to recover data */
        /* hopefully, verseed will not be around by 2010 */
 
        if (rec_time.year < 1950 ||
                        rec_time.year > 2010)
        {
                rec_time.year = swap_2byte(rec_time.year);
                rec_time.day  = swap_2byte(rec_time.day);
                rec_time.fracsec = swap_2byte(rec_time.fracsec);
        }
 
	get_stn_chn_rec(s, c, n, l, &rec_time);

        if ((current_station == NULL) || (current_channel == NULL))
        {
                get_stn_chn_rec(s, c, n, l, NULL);

		if ((current_station == NULL) || (current_channel == NULL))
                        return 4096;
        }
 
        return 2 << (current_channel->log2drecl - 1);
 
}

/* -------------------------------------------------------------------- */
char *scan_for_blk_1000();
/* -------------------------------------------------------------------- */

int get_blk_1000_Lrecl(data_hdr, default_lrecl)
struct input_data_hdr *data_hdr;
int default_lrecl;
 
 
{
        int num_bytes;

        char station[20];
        char channel[20];
        char network_code[5];
        char loc[5];

	char tmp_str[255];
 
        struct data_blk_1000 *p;

        strncpy(station, data_hdr->station, sizeof(data_hdr->station));
        station[sizeof(data_hdr->station)] = 0;
        trim(station);

        strncpy(channel, data_hdr->channel, sizeof(data_hdr->channel));
        channel[sizeof(data_hdr->channel)] = 0;
        trim(channel);

        strncpy(network_code, data_hdr->network, sizeof(data_hdr->network));
        network_code[sizeof(data_hdr->network)] = 0;
        trim(network_code);

        strncpy(loc, data_hdr->location, sizeof(data_hdr->location));
        loc[2] = 0;

        trim(loc);

        /* sanity check */
        if (((data_hdr->bofb > 0) && (data_hdr->bofb < 48)) || (data_hdr->bofb > 96))
        {
		char tmp_str[255];

		sprintf(tmp_str, "ERROR: Unable to scan for blk 1000. Bad bofb entry! Found:%d", data_hdr->bofb);

		log_tspan_error(station,
                                channel,
                                network_code,
                                loc,
                                &(data_hdr->time),
                                tmp_str);

		if (sniff_flag > 0)
		{
			fprintf(stderr, "Warning: Bad bofb entry, scanned:%d\n", data_hdr->bofb);

			fprintf(stderr, "for stn/chn/loc/net:%s/%s/%s/%s",
					station,
					channel,
					loc,
					network_code);

			fprintf(stderr, "\tdata rec time:%d,%d,%d:%d:%d.%d\n",
					data_hdr->time.year,
					data_hdr->time.day,
					data_hdr->time.hour,
					data_hdr->time.minute,
					data_hdr->time.second,
					data_hdr->time.fracsec);

		}
	
                return default_lrecl;

        }

	if((data_hdr->number_blockettes == 0) && 
		(data_hdr->bofb != 0))
	{
		if (sniff_flag == 0)
		{
			sprintf(tmp_str, "WARNING: data header number of blockettes = %d, data header beginning of blockettes=%d", data_hdr->number_blockettes, data_hdr->bofb);

               		log_tspan_error(station,
                                	channel,
                                	network_code,
                                	loc,
                                	&(data_hdr->time),
					tmp_str);
		}
		else
		{
			fprintf(stderr, "ERROR: data header number of blockettes = %d, data header beginning of blockettes=%d ", data_hdr->number_blockettes, data_hdr->bofb);

			fprintf(stderr, "for %s, %s, %s, %s at %d,%d,%d:%d:%d.%d\n",
					station,
                                        channel,
                                        network_code,
                                        loc, 
					data_hdr->time.year,
					data_hdr->time.day,
					data_hdr->time.hour,
					data_hdr->time.minute,
					data_hdr->time.second,
					data_hdr->time.fracsec);
		}
	}
	

        if (data_hdr->bofb != 0)
                p = (struct data_blk_1000 *)
			scan_for_blk_1000((((char *)data_hdr) + 
						data_hdr->bofb) - 8,
                                                (char *)data_hdr - 8);
        else
                p = 0;
 
        if (p == NULL)
                return default_lrecl;

	/* check for sanity of wordorder in blk1000 */
        if (p->word_order > 1)
        {

		/* Kludge Alert: some data has garbage in the
		 * word order field, but also has a number_of_blockettes
		 * field of zero, which is harmless via rdseed.
		 * So no error if this is the case.
		 */

		if (data_hdr->number_blockettes > 0)
		{
			sprintf(tmp_str, 
				"ERROR: Bad blk 1000 word order detected. Found: %d", p->word_order);
                	log_tspan_error(station,
                                	channel,
                                	network_code,
                                	loc,
                                	&(data_hdr->time),
					tmp_str);
		}
	
		if (sniff_flag > 0)
		{
			/* always report error if sniff level > 0 */

			fprintf(stderr, "ERROR: Bad block 1000 word order entry, scanned:%d\n", p->word_order);

			fprintf(stderr, "for stn/chn/loc/net:%s/%s/%s/%s",
					station,
					channel,
					loc,
					network_code);

			fprintf(stderr, "\tdata rec time:%d,%d,%d:%d:%d.%d\n",
					data_hdr->time.year,
					data_hdr->time.day,
					data_hdr->time.hour,
					data_hdr->time.minute,
					data_hdr->time.second,
					data_hdr->time.fracsec);
		}
        }
 
        return 2 << (p->rec_length - 1);
        
}
 
/* --------------------------------------------------------------------- */
char *scan_for_blk_1000(b_ptr, base)
struct data_blk_hdr *b_ptr;
char * base;        /* start of logical rec */

{

        while (1)
        {
                if (b_ptr->type == 1000)
                        /* eureka, we've found it */
                        return (char *)b_ptr;

                if (b_ptr->next_blk_byte == 0)
                        return (char *) NULL;

                /* garbage check */
                switch (b_ptr->type)
                {
                        case 100 :
				break;
                        case 201:
				break;

                        case 300:
                        case 310:
                        case 200:
                        case 320:
                        case 390:
                        case 395:
                        case 400:
                        case 405:
                        case 500:
                                break;
                        default : /* oh, oh */
         
                                fprintf(stderr,
"scan_for_blockette_1000(): Bad blockette scanned\n Blockette = %d\n", b_ptr->type);
                        
                                return (char *) 0;
 
                }               /* switch */
        
                b_ptr = (struct data_blk_hdr *)(base + b_ptr->next_blk_byte);
 
        }   /* while */
 
        /* Should never get here */
        return (char *) 0;
 
}
 
/* -------------------------------------------------------------------- */


