/*===========================================================================*/
/* SEED reader     |               parse_type50              |  station header */
/*===========================================================================*/
/*
	Name:		parse_type50
	Purpose:	parse a station id (type 50) blockette into a 
				globally-available table
	Usage:		void parse_type50 ();
				char *blockette;
				parse_type50 (blockette);
	Input:		blockette = pointer to beginning of blockette
	Output:		none
	Externals:	temp_char
				type50_head - 1st entry in table; defined in structures.h,
					allocated in globals.h
	Warnings:	none
	Errors:		none
	Called by:	process_abbrevdic
	Calls to:	parse_int - get an integer from a blockette 
				parse_double - get a double from a blockette 
				parse_nchar - get a fixed-length string from a blockette 
				parse_varlstr - get a variable-length string from a blockette
	Algorithm:	parse the blockette
	Notes:		none
	Problems:	none known
	References:	Halbert et al, 1988; see main routine
	Language:	C, hopefully ANSI standard
	Author:		Dennis O'Neill
	Revisions:	07/15/88  Dennis O'Neill  Initial preliminary release 0.9
				11/04/88  Dennis O'Neill  replaced "~" by STRINGTERM in
				                          parse_varlstr calls
				11/21/88  Dennis O'Neill  Production release 1.0
				08/19/89  Dennis O'Neill  streamlined linked list construction
				09/17/89  Dennis O'Neill  removed number_channels and 
				                          number_comments fields
*/

#include "rdseed.h"

/* this needs to be allocated here so it keeps its value between calls */
struct type50 *type50_tail = NULL;					/* end of linked list */

void parse_type50 (blockette)
char *blockette;							/* ptr to start of blockette */
{
	char *blockette_ptr;						/* ptr to inside blockette */
	struct type50 *type50; 						/* genl member of table */
	struct time begin_time, end_time;
	int    cmp_error;

	/* point to beginning of information, past type and length */
	blockette_ptr = blockette;
	blockette_ptr += 7;

/*                 +=======================================+                 */
/*=================| allocate space and initialize a type50  |=================*/
/*                 +=======================================+                 */

	/* allocate space for a type50 record */
	type50 = (struct type50 *) alloc_linklist_element (sizeof (struct type50),
		"parse_type50");	

	/* set the global variable current_station to this record */
	current_station = type50;

/*                 +=======================================+                 */
/*=================|   recover information from the type50   |=================*/
/*                 +=======================================+                 */

	/* recover station symbol */
	type50->station = parse_nchar (&blockette_ptr, 5);

	/* recover latitude */
	type50->latitude = parse_double (&blockette_ptr, 10);

	/* recover longitude */
	type50->longitude = parse_double (&blockette_ptr, 11);

	/* recover elevation */
	type50->elevation = parse_double (&blockette_ptr, 7);

	/* recover number of channels - old spec, removed */
	/* type50->number_channels = parse_int (&blockette_ptr, 4); */
	type50->reserved1 = parse_int (&blockette_ptr, 4);

	/* recover number of station comment blockettes - old spec, removed */
	/* type50->number_comments = parse_int (&blockette_ptr, 3); */
	type50->reserved2 = parse_int (&blockette_ptr, 3);

	/* recover site name */
	type50->name = parse_varlstr (&blockette_ptr, STRINGTERM);

	/* recover network/station owner name code (see generic abbrev dict) */
	type50->owner_code = parse_int (&blockette_ptr, 3);

	/* recover 32-bit byte significance order */
	type50->longword_order = parse_int (&blockette_ptr, 4);

	/* recover 16-bit byte significance order */
	type50->word_order = parse_int (&blockette_ptr, 2);

	/* recover starting effective date */
	type50->start = parse_varlstr (&blockette_ptr, STRINGTERM);

	/* recover ending effective date, allowing for zero length */
	if (*blockette_ptr != '~')
		type50->end = parse_varlstr (&blockette_ptr, STRINGTERM);
	else
	{
		blockette_ptr += 1;
		type50->end = NULL;
	}

	timecvt(&begin_time, type50->start);
	timecvt(&end_time  , type50->end);
	type50->start_time = begin_time;
	type50->end_time   = end_time;

	cmp_error = FALSE;
	if (timecmp(begin_time, begin_vol_time) < 0)
	{
		if (timecmp(end_time, begin_vol_time) < 0)  cmp_error = TRUE;
	}
	else if (timecmp(begin_time, end_vol_time) > 0) cmp_error = TRUE;

	if (cmp_error) response_error++;

	if (sniff_flag >= 2 || (cmp_error && (sniff_flag >= 1)))
	{
		fprintf(stderr, "Station %s Effective Time ", type50->station);
		timeprt(begin_time);
		fprintf(stderr, " to ");
		timeprt(end_time);
		fprintf(stderr, " ");
		if (cmp_error) fprintf(stderr, "<=== ERROR\n");
		else fprintf(stderr, "\n");
	}

	/* recover update flag */
	type50->update = parse_nchar (&blockette_ptr, 1);

	/* recover network flag */
    if (type10.version >= 2.3)
    {
        type50->network_code = parse_nchar (&blockette_ptr, 2);
    }
    else
    {   
        type50->network_code = NULL;
    }

	/* initialize the pointer for the first of each of several elements */
	type50->type51_head = NULL;
	type50->type51_tail = NULL;
	type50->type52_head = NULL;
	type50->type52_tail = NULL;
	type50->station_update = NULL;

/*                 +=========================================+                 */
/*=================|   link the type50 into the type50 table |=================*/
/*                 +=========================================+                 */

	append_linklist_element (type50, type50_head, type50_tail);

}
