/*===========================================================================*/
/* SEED reader     |               parse_type52              |  station header */
/*===========================================================================*/
/*
	Name:		parse_type52
	Purpose:	parse a channel id (type 52) blockette into a globally-available
				table
	Usage:		void parse_type52 ();
				char *blockette;
				parse_type52 (blockette);
	Input:		blockette = pointer to beginning of blockette
	Output:		none
	Externals:	temp_char
				type52_head - 1st entry in table; defined in structures.h,
					allocated in globals.h
	Warnings:	none
	Errors:		none
	Called by:	process_stationh
	Calls to:	parse_int - get an integer from a blockette 
				parse_long - get a long integer from a blockette 
				parse_double - get a double from a blockette 
				parse_nchar - get a fixed-length string from a blockette 
				parse_varlstr - get a variable-length string from a blockette
	Algorithm:	parse the blockette into a globally-available table
	Notes:		none
	Problems:	none known
	References:	Halbert et al, 1988; see main routine
	Language:	C, hopefully ANSI standard
	Author:		Dennis O'Neill
	Revisions:	07/15/88  Dennis O'Neill  Initial preliminary release 0.9
				11/04/88  Dennis O'Neill  replaced "~" by STRINGTERM in
				                          parse_varlstr calls
				11/21/88  Dennis O'Neill  Production release 1.0
				08/19/89  Dennis O'Neill  streamlined linked list construction
				09/17/89  Dennis O'Neill  removed number_ch_comments
*/

#include "rdseed.h"

struct type50 *find_station_rec();

/* ------------------------------------------------------------------------- */


void parse_type52 (blockette)
char *blockette;							/* ptr to start of blockette */
{
	char *blockette_ptr;						/* ptr to inside blockette */
	struct type52 *type52; 						/* genl member of table */
	struct time begin_time, end_time;
	int cmp_error, overlap_error;

	struct type50 *ptr;
    struct time begin_t;
    struct time end_t;

	/* point to beginning of information, past type and length */
	blockette_ptr = blockette;
	blockette_ptr += 7;


/*                 +=======================================+                 */
/*=================| allocate space and initialize a type52  |=================*/
/*                 +=======================================+                 */

	type52 = (struct type52 *) alloc_linklist_element (sizeof (struct type52),
		"parse_type52");	

	/* set the global variable current_channel to this record */
	current_channel = type52;

/*                 +=======================================+                 */
/*=================|   recover information from the type52   |=================*/
/*                 +=======================================+                 */

	/* recover location identifier */
	type52->location = parse_nchar (&blockette_ptr, 2);

	/* recover channel identifier */
	type52->channel = parse_nchar (&blockette_ptr, 3);

	/* recover subchannel identifier (for multiplexing) */
	type52->subchannel = parse_int (&blockette_ptr, 4);

	/* recover index for instrument ident in dictionary */
	type52->instrument_code = parse_int (&blockette_ptr, 3);

	/* recover optional instrument comment */
	if (*blockette_ptr != '~')
		type52->inst_comment = parse_varlstr (&blockette_ptr, STRINGTERM);
	else
	{
		type52->inst_comment = NULL;
		blockette_ptr += 1;
	}

	/* recover index for signal units in dictionary */
	type52->signal_units_code = parse_int (&blockette_ptr, 3);

	/* recover index for calibration units in dictionary */
	type52->calib_units_code = parse_int (&blockette_ptr, 3);

	/* recover latitude */
	type52->latitude = parse_double (&blockette_ptr, 10);

	/* recover longitude */
	type52->longitude = parse_double (&blockette_ptr, 11);

	/* recover elevation */
	type52->elevation = parse_double (&blockette_ptr, 7);

	/* recover local depth */
	type52->local_depth = parse_double (&blockette_ptr, 5);

	/* recover azimuth */
	type52->azimuth = parse_double (&blockette_ptr, 5);

	/* recover dip */
	type52->dip = parse_double (&blockette_ptr, 5);

	/* recover data format id lookup code */
	type52->format_code = parse_int (&blockette_ptr, 4);

	/* recover log2 of data record length */
	type52->log2drecl = parse_int (&blockette_ptr, 2);

	/* recover sample rate */
	type52->samplerate = parse_double (&blockette_ptr, 10);

	/* recover maximum clock drift tolerance */
	type52->clock_tolerance = parse_double (&blockette_ptr, 10);

	/* recover number of channel comment blockettes - old spec, reserved */
	/* type52->number_ch_comments = parse_int (&blockette_ptr, 4); */
	type52->reserved1 = parse_int (&blockette_ptr, 4);

	/* recover channel flags, if any */
	if (*blockette_ptr != '~')
		type52->channel_flag = parse_varlstr (&blockette_ptr, STRINGTERM);
	else blockette_ptr += 1;

	/* recover effective start date */
	type52->start = parse_varlstr (&blockette_ptr, STRINGTERM);

	/* recover effective end date, if any */
	if (*blockette_ptr != '~')
		type52->end = parse_varlstr (&blockette_ptr, STRINGTERM);
	else
	{
		blockette_ptr += 1;
		type52->end = NULL;
	}

	timecvt(&begin_time, type52->start);
	timecvt(&end_time  , type52->end);
	type52->start_time = begin_time;
	type52->end_time   = end_time;

	overlap_error = FALSE;
	{
		struct type52 *t52; 					/* genl member of table */

		for (t52 = current_station->type52_head;
				t52 != NULL;
				t52 = t52->next)
		{
			if (strcmp(type52->channel, t52->channel) != 0) continue;
			if (strcmp(type52->location, t52->location) != 0) continue;
                         
			if (((timecmp(begin_time, t52->start_time) >= 0) &&
				(timecmp(begin_time, t52->end_time) < 0)) ||
				((timecmp(begin_time, t52->start_time) < 0) &&
				(timecmp(end_time, t52->start_time) > 0)) )
			{
				overlap_error = TRUE;
			}
		}
	}	

	cmp_error = FALSE;
	if (timecmp(begin_time, begin_vol_time) < 0)
		{
		if (timecmp(end_time, begin_vol_time) < 0)  cmp_error = TRUE;
		}
	if (timecmp(begin_time, end_vol_time) > 0) cmp_error = TRUE;

	if (cmp_error) response_error++;
	if (overlap_error) response_overlap_error++;

	if (sniff_flag >= 2 || ((cmp_error||overlap_error) && (sniff_flag >= 1)))
	{
		fprintf(stderr, "\tChannel %s,%s Effective Time ", type52->location, type52->channel);
		timeprt(begin_time);
		fprintf(stderr, " to ");
		timeprt(end_time);
		fprintf(stderr, " ");
		if (cmp_error) fprintf(stderr, "<== RANGE");
		if (overlap_error) fprintf(stderr, "<== OVERLAP");
		fprintf(stderr, "\n");
	}
	/* recover update flag */
	type52->update = parse_nchar (&blockette_ptr, 1);

	/* initialize the pointer for the first of each of several elements */
	type52->response_head = NULL;
	type52->response_tail = NULL;
	type52->type59_head = NULL;
	type52->type59_tail = NULL;
	type52->channel_update = NULL;

/*                 +=======================================+                 */
/*=================|   link the type52 into the type31 table   |=================*/
/*                 +=======================================+                 */

	/* find current station from station list */
 	timecvt(&begin_t, type52->start);
    	timecvt(&end_t, type52->end);
   
    	/* save current if not found */
    	ptr = current_station;
 
    	current_station = find_station_rec(current_station->station,
                                        	current_station->network_code,
                                        	&begin_t, &end_t);
 
    	/* if not found, better warn the user ! */
    	if (current_station == NULL)
    	{
        	fprintf(stderr, "WARNING - Possible date conflict between the station effective times and this channel.\nCurrent station: %s - network:%s\n", ptr->station, ptr->network_code ? ptr->network_code : "N/A");
 
        	current_station = ptr;

		/* dump effective times to screen for visual feedback */
		dump_station_effective(current_station->station, current_station->network_code);

		fprintf(stderr, "channel %s effective start/stop times:\n\t\t%s / %s\n",
				type52->channel,
				type52->start == NULL ? 
					"No starting time": type52->start,
				type52->end == NULL ? 
					"No ending time" : type52->end);

    	}
 
	append_linklist_element (type52, current_station->type52_head,
		current_station->type52_tail);

}
