/*===========================================================================*/
/* SEED reader     |               parse_type53              |  station header */
/*===========================================================================*/
/*
	Name:		parse_type53
	Purpose:	parse a response (poles and zeroes) (type 53) blockette
				into a globally-available table
	Usage:		void parse_type53 ();
				char *blockette;
				parse_type53 (blockette);
	Input:		blockette = pointer to beginning of blockette
	Output:		none
	Externals:	temp_char
				type53_head - 1st entry in table; defined in structures.h,
					allocated in globals.h
	Warnings:	none
	Errors:		none
	Called by:	process_stationh
	Calls to:	parse_int - get an integer from a blockette 
				parse_double - get a double from a blockette 
				parse_nchar - get a fixed-length string from a blockette 
	Algorithm:	parse the blockette into a globally-available table
	Notes:		none
	Problems:	none known
	References:	Halbert et al, 1988; see main routine
	Language:	C, hopefully ANSI standard
	Author:		Dennis O'Neill
	Revisions:	07/15/88  Dennis O'Neill  Initial preliminary release 0.9
				11/21/88  Dennis O'Neill  Production release 1.0
				08/19/89  Dennis O'Neill  streamlined linked list construction
				06/25/93  C. Laughbon     added check for conjugate pole pairs
										  plus misc pole & zero checking

*/
#include "rdseed.h"
#include <errno.h>

char pole_pairs_matched();
char *strtime();

void parse_type53 (blockette)
char *blockette;								/* ptr to start of blockette */
{
	char *blockette_ptr;						/* ptr to inside blockette */
	struct type53 *type53; 							/* genl member of table */
	int i;										/* counter */
	struct response *response;					/* channel response ptr */

	/* point to beginning of information, past type and length */
	blockette_ptr = blockette;
	blockette_ptr += 7;

/*                 +=======================================+                 */
/*=================| allocate space and initialize a type53  |=================*/
/*                 +=======================================+                 */

	type53 = (struct type53 *) alloc_linklist_element (sizeof (struct type53),
		"parse_type53");	

/*                 +=======================================+                 */
/*=================|   recover information from the type53   |=================*/
/*                 +=======================================+                 */

	/* recover transfer function type */
	type53->transfer = parse_nchar (&blockette_ptr, 1);

	/* recover cascade sequence number */
	type53->stage = parse_int (&blockette_ptr, 2);

	/* recover response in units (0 if none) */
	type53->input_units_code = parse_int (&blockette_ptr, 3);

	/* recover response out units */
	type53->output_units_code = parse_int (&blockette_ptr, 3);

	/* recover AO normalization factor (1.0 if none) */
	type53->ao_norm = parse_double (&blockette_ptr, 12);

	/* recover normalization frequency */
	type53->norm_freq = parse_double (&blockette_ptr, 12);

    if (!str_isnum(blockette_ptr, 3))
    { 
        fprintf(stderr, "Warning - non-numeric field scanned for blockette 53's # zeros! Station: %s, channel: %s, network: %s\n",
                    current_station->station,  
                    current_channel->channel, 
                    type10.version >= 2.3 ? current_station->network_code : "N/A");  
 
    }  

	/* recover number of complex zeroes */
	type53->number_zeroes = parse_int (&blockette_ptr, 3);

	/* allocate space for complex zeroes */
	type53->zero = (struct type53sub *) alloc_linklist_element 
		(sizeof (struct type53sub) * type53->number_zeroes, "parse_type53->zero");

	/* recover complex zeroes */
	for (i = 0; i < type53->number_zeroes; i++)
	{
		type53->zero[i].real = parse_double (&blockette_ptr, 12);
		type53->zero[i].imag = parse_double (&blockette_ptr, 12);
		type53->zero[i].real_error = parse_double (&blockette_ptr, 12);
		type53->zero[i].imag_error = parse_double (&blockette_ptr, 12);
	}

    if (!str_isnum(blockette_ptr, 3))
    { 
        fprintf(stderr, "Warning - non-numeric field scanned for blockette 53's # poles!\nStation: %s, channel: %s, network: %s\n",
                    current_station->station,  
                    current_channel->channel, 
                    type10.version >= 2.3 ? current_station->network_code : "N/A");  
 
    }  
 
	/* recover number of complex poles */
	type53->number_poles = parse_int (&blockette_ptr, 3);

	/* allocate space for complex poles */
	type53->pole = (struct type53sub *) alloc_linklist_element
		(sizeof (struct type53sub) * type53->number_poles, "parse_type53->pole");

	/* recover complex poles */
	for (i = 0; i < type53->number_poles; i++)
	{
		type53->pole[i].real = parse_double (&blockette_ptr, 12);
		type53->pole[i].imag = parse_double (&blockette_ptr, 12);
		type53->pole[i].real_error = parse_double (&blockette_ptr, 12);
		type53->pole[i].imag_error = parse_double (&blockette_ptr, 12);
	}

/*                +==========================================+            */
/*================|   link the type53 into the type53 table  |============*/
/*                +==========================================+            */

	/* initialize the pointer to the next record as end-of-list marker */
	type53->next = NULL;

	if (look_response('P', type53->stage))
	{
		response_error++;
		if (sniff_flag >= 1) fprintf(stderr, "ERROR - Duplicate Blockette 53 stage %d, %s,%s\n", type53->stage, current_station->station, current_channel->channel);
	}

	response = get_response ('P');
	response->ptr.type53 = type53;	
}

/* -------------------------------------------------------------------- */
#define E_POLE_GT_0 1
#define E_POLE_NOT_MATCHED 2
#define E_ZEROPOLE_WO_ZERO 4
#define E_NUMPOLES_LT_NUMZEROS 8

static char err_flag = 0;

static int num_gt_zero_errors = 0;
static int num_unmatched_poles_errors = 0;
static int num_zeropoles_wo_zero_errors = 0;
static int num_poles_LT_num_zeros = 0;


/* ----------------------------------------------------------------------- */

char pole_pairs_matched(pole_array, num_poles, zero_array, num_zeros)
struct  type53sub *pole_array;
struct  type53sub *zero_array;
int num_zeros;
int num_poles;

{
    int i, ix;
	char *pole_flag;
	char *zero_flag;


	err_flag = 0;

	pole_flag = (char *)malloc(num_poles);

	if (!pole_flag)
	{
		fprintf(stderr, "Unable to malloc - code error. Can't check poles\n");
		return 0;	/* no error */
	}

	memset(pole_flag, 0, num_poles);

	zero_flag = (char *)malloc(num_zeros);

	if (!pole_flag) 
    { 
        fprintf(stderr, "Unable to malloc - code error. Can't check zeros\n"); 
        return 0;   /* no error */
    }
 
    memset(zero_flag, 0, num_zeros);

	for (i = 0; i < num_poles; i++)
	{
		/* if already flagged as found...move on */
		if (pole_flag[i]) 
			continue;

		/* make sure real is > 0 */
		/*
		if (pole_array[i].real > 0)
		{
			err_flag |= E_POLE_GT_0;
			num_gt_zero_errors++;

		}
		*/
		/* make sure that is pole == 0, there is a corresponding zero */
		if (pole_array[i].real == 0)
		{
			/* scan zero table for a 0.0 */
			for (ix = 0; ix < num_zeros; ix++)
			{
					/* if value is zero AND unused table entry */
				if (zero_array[ix].real == 0 && (!zero_flag[ix]))
				{
					zero_flag[ix] = TRUE;
				
					break;	
				}
			}		/* for ix */

            /* if ix == num_zeros, then we didn't find it in list - 
		     *  flag error 
			 */
			if (ix == num_zeros)
			{
				err_flag |= E_ZEROPOLE_WO_ZERO;
				num_zeropoles_wo_zero_errors++;
			}

			/* done with this pole - continue thru list */
			pole_flag[i] = TRUE;
			continue;

		}			/* if pole == 0 */


		if (pole_array[i].imag == 0)
		{
			pole_flag[i] = TRUE;

			continue;   /* just go on - 
						 * no pairing possible with zero imaginary number
					 	 */
		}

		for (ix = i + 1; ix < num_poles; ix++)

			if (pole_array[i].real == pole_array[ix].real)
			{	/* found matching real */
											
				/* check for negative reciprical */
				if (pole_array[i].imag != (-pole_array[ix].imag))
					continue;

				/* flag both poles as found */
				pole_flag[i] = pole_flag[ix] = TRUE;

				break;		/* don't continue with list if found (ix loop)*/

			}		/* if pole[1] == -pole[ix] */	

	}		/* for i */

	/* scan the flag array for unmatched poles */
    for (i = 0; i < num_poles; i++)
		if (!pole_flag[i])
		{
			err_flag |= E_POLE_NOT_MATCHED;
			num_unmatched_poles_errors++;
		}

	free(pole_flag);
	free(zero_flag);

	return err_flag;

}

void print_error_message(error_byte)

{

	if (error_byte & E_NUMPOLES_LT_NUMZEROS) 
	{
		fprintf(stderr, "WARNING - Found %d responses with number of poles less the number of zeros - ", num_poles_LT_num_zeros);
 
   	} 

	if (error_byte & E_POLE_NOT_MATCHED)
    {  
		fprintf(stderr, 
			    "WARNING! Found %d pole(s) without corresponding pair - ", 
                num_unmatched_poles_errors);

    }
	if (error_byte & E_POLE_GT_0) 
	{ 
		fprintf(stderr, "WARNING - Found %d pole(s) with real value > 0.0 - ",
						  num_gt_zero_errors);

	} 
	if (error_byte & E_ZEROPOLE_WO_ZERO) 
	{ 
		fprintf(stderr, "WARNING - Found %d pole(s) with a real value equaling zero, without a corresponding zero - ", num_zeropoles_wo_zero_errors);
 
	}

	return;
}

/* ------------------------------------------------------------------------ */
void process_error(error_byte, ptr_to_50, ptr_to_52)
char error_byte;
struct type50 *ptr_to_50;
struct type52 *ptr_to_52;

{

	char start_time_buf[200];
	char end_time_buf[200];

	if (!error_byte) return;

	print_error_message(error_byte);

	strcpy(start_time_buf, strtime(start_time_buf, &(ptr_to_52->start_time)));
	strcpy(end_time_buf, strtime(end_time_buf, &(ptr_to_52->end_time)));

	fprintf(stderr, "\tNetwork:  %s, Station: %s; channel: %s\tStart time: %s\tEnding time: %s\n", 
						ptr_to_50->network_code,ptr_to_50->station,ptr_to_52->channel,start_time_buf,end_time_buf);

	num_poles_LT_num_zeros = num_unmatched_poles_errors = num_gt_zero_errors = num_zeropoles_wo_zero_errors = 0;

	return;
}

/* ------------------------------------------------------------------------- */
char qa_type53(ptr)
struct type53 *ptr;

{
	char error_byte;
	
	/* do some QA */
	/* check complex conjugate pairs for opposite signs, etc  */

	error_byte = pole_pairs_matched(ptr->pole, ptr->number_poles,
								    ptr->zero, ptr->number_zeroes); 

	/* number of zeros should be less than or equal to the poles */
	if (ptr->number_poles < ptr->number_zeroes)
	{
		num_poles_LT_num_zeros++;
        
		error_byte |= E_NUMPOLES_LT_NUMZEROS;

	}

	return error_byte;

}

/* ---------------------------------------------------------------- */
void qa_type43(ptr)
struct type43 *ptr;

{
    char error_byte;
    
    /* do some QA */
    /* check complex conjugate pairs for opposite signs, etc  */

    error_byte = pole_pairs_matched(ptr->pole, ptr->number_poles,
                                    ptr->zero, ptr->number_zeroes);

    /* number of zeros should be less than or equal to the poles */
    if (ptr->number_poles < ptr->number_zeroes)
    {
        num_poles_LT_num_zeros++;
       
        error_byte |= E_NUMPOLES_LT_NUMZEROS;

    }

	if (error_byte)
	{
		char station[10], channel[10], network[10];

		print_error_message(error_byte);

		if (find_station_channel(ptr->response_code, type50_head,
							 station, channel, network))
		{
			fprintf(stderr, "for network %s ; station: %s  ; channel: %s\n", network, station, channel);
		}
		else
		{
			fprintf(stderr, "No station and/or channel information available\n");
		}

	}


    return;

}

/* ------------------------------------------------------------------ */
int find_station_channel(code, ptr_type50, station, channel, network)
int code;
struct type50 *ptr_type50;
char *station, *channel;
char *network;

{
	struct type52 *ptr_type52;
	struct type60 *ptr_type60;

	while (ptr_type50)
	{
	
		ptr_type52 = ptr_type50->type52_head;

		if (ptr_type52 == NULL)
		{
			fprintf(stderr, "Warning - unable to find channel information for station %s!\n", ptr_type50->station);

		}
		else
			if (scan_responses(code, ptr_type52->response_head))
			{
				strcpy(channel, ptr_type52->channel);
				strcpy(station, ptr_type50->station);
				if (type10.version >= 2.3)
					strcpy(network, ptr_type50->network_code);
				else
					strcpy(network, "");
	
				return 1;
			}

		ptr_type50 = ptr_type50->next;
	}

	return 0;

}

int scan_responses(code, r)
int code;
struct response *r;

{
	while (r)
	{
		if (r->type == 'R')
			if (scan_stages(code, r->ptr.type60))
				return(1);	

		r = r->next;
		
	}

	return(0);

}

int scan_stages(code, ptr_type60)
int code;
struct type60 *ptr_type60;

{
	int i, ix;
	
	while (ptr_type60)
	{
		for (i = 0; i < ptr_type60->number_stages; i++)
		{
			if (scan_resp(code, ptr_type60->stage + i))
				return(1);
	
		}
		
		ptr_type60 = ptr_type60->next;

	}

	return(0);
	
}

int scan_resp(code, stage)
int code;
struct  type60sub1 *stage;

{
	int i;

	for (i = 0; i < stage->number_responses; i++)
	{
		if (code == (stage->response + i)->reference)
		{
			/* whew - found it */
			return(1);
		}

	}

	/* all this work and we didn't find it - ugh */
	return(0);
}

