/*===========================================================================*/
/* SEED reader     |            parse_type74             |     time header */
/*===========================================================================*/
/*
	Name:		parse_type74
	Purpose:	parse a time series index (type 74) blockette into a
			globally-available table
	Usage:		void parse_type74 ();
			char *blockette;
			parse_type74 (blockette);
	Input:		blockette = pointer to beginning of blockette
	Output:		none
	Externals:	temp_char
			type74_head - 1st entry in table; defined in structures.h,
			allocated in globals.h
	Warnings:	none
	Errors:		none
	Called by:	process_stationh
	Calls to:	parse_int - get an integer from a blockette 
			parse_double - get a double from a blockette 
			parse_varlstr - get a variable-length string from a blockette
	Algorithm:	parse the blockette into a globally-available table
	Notes:		none
	Problems:	none known
	References:	Halbert et al, 1988; see main routine
	Language:	C, hopefully ANSI standard
	Author:		Dennis O'Neill
	Revisions:	08/29/89  Dennis O'Neill  Initial version
*/

#include "rdseed.h"

/* this needs to be allocated here so it keeps its value between calls */
struct type74 *type74_tail = NULL;			/* end of linked list */

void parse_type74 (blockette)
char *blockette;			/* ptr to start of blockette */
{
	char *blockette_ptr;		/* ptr to inside blockette */
	struct type74 *type74; 		/* genl member of table */
	struct type52 *type52;
	int i, cmp_error, cmp1_error, cmp2_error;	/*counter */
	struct time begin_time, end_time, accel_time;

	/* point to beginning of information, past type and length */
	blockette_ptr = blockette;
	blockette_ptr += 7;

/*                 +=======================================+                 */
/*=================| allocate space and initialize a type74|=================*/
/*                 +=======================================+                 */

	type74 = (struct type74 *) alloc_linklist_element
		(sizeof (struct type74), "parse_type74");	

/*                 +=======================================+                 */
/*=================|  recover information from the type74  |=================*/
/*                 +=======================================+                 */

	memset((char *)type74, sizeof(struct type74), 0);

	type74->station = parse_nchar (&blockette_ptr, 5);
	type74->location = parse_nchar (&blockette_ptr, 2);
	type74->channel = parse_nchar (&blockette_ptr, 3);

	if (*blockette_ptr != '~')
		type74->starttime = parse_varlstr (&blockette_ptr, STRINGTERM);
	else blockette_ptr += 1;
	type74->start_index = parse_long (&blockette_ptr, 6);
	type74->start_subindex = parse_int (&blockette_ptr, 2);

	if (*blockette_ptr != '~')
		type74->endtime = parse_varlstr (&blockette_ptr, STRINGTERM);
	else blockette_ptr += 1;
	type74->end_index = parse_long (&blockette_ptr, 6);
	type74->end_subindex = parse_int (&blockette_ptr, 2);

	timecvt(&begin_time, type74->starttime);
	timecvt(&end_time  , type74->endtime);

	cmp_error = FALSE;
	if (timecmp(begin_time, current_start_time_span) < 0) {cmp_error = TRUE;}
	if (timecmp(begin_time, current_end_time_span) > 0) {cmp_error = TRUE;}
	if (timecmp(end_time, current_start_time_span) < 0) {cmp_error = TRUE;}
/*	if (timecmp(end_time, current_end_time_span) > 0) cmp_error = TRUE; */

	number_of_blk74++;		/* Count number of blockette 74's */

	if (cmp_error) 
		time_span_error++;

	cmp1_error = cmp2_error = TRUE;	/* Guilty until found innocent */
	for (current_station = type50_head; current_station != NULL;
			current_station = current_station->next)
	{
		if (timecmp(begin_time, current_station->start_time) >= 0)
			if (timecmp(begin_time, current_station->end_time) <= 0) cmp1_error = FALSE;

		for (type52 = current_station->type52_head; type52 != NULL; type52=type52->next)
		{
			if (timecmp(begin_time, type52->start_time) >= 0)
				if (timecmp(begin_time, type52->end_time) <= 0)
			{
				cmp2_error = FALSE;
				break;
			}
		}
		if (!cmp1_error && !cmp2_error) break;
	}

	if (sniff_flag >= 2 || ((cmp_error||cmp1_error||cmp2_error) && (sniff_flag >= 1)))
	{
		fprintf(stderr, "\t%s.%s.%s Time Series ", type74->station, type74->location, type74->channel);
		timeprt(begin_time);
		fprintf(stderr, " to ");
		timeprt(end_time);
		fprintf(stderr, "  ");
/*		fprintf(stderr, "Record %d to %d ", type74->start_index, type74->end_index); */

		if (cmp_error||cmp1_error||cmp2_error) fprintf(stderr, "<=== TIME ERROR\n");
		else fprintf(stderr, "\n");
	}

	type74->number_accelerators = parse_int (&blockette_ptr, 3);	/* recover number of accelerators */

	/* allocate space for data pieces */
	type74->accelerator = (struct type74sub *) alloc_linklist_element
		(sizeof (struct type74sub) * type74->number_accelerators,
		"parse_type74->datapiece");

	/* recover accelerators */
	for (i = 0; i < type74->number_accelerators; i++)
	{
		if (*blockette_ptr != '~')
			type74->accelerator[i].time =
			parse_varlstr (&blockette_ptr, STRINGTERM);
		else
		{
			blockette_ptr += 1;
			type74->accelerator[i].time = NULL;
		}
		type74->accelerator[i].index = parse_long (&blockette_ptr, 6);
		type74->accelerator[i].subindex = parse_int (&blockette_ptr, 2);

		timecvt(&accel_time, type74->accelerator[i].time);

		cmp_error = FALSE;
		if (timecmp(accel_time, begin_time) < 0) cmp_error = TRUE;
		if (timecmp(accel_time, end_time) > 0) cmp_error = TRUE;

		if (cmp_error) 
			time_span_error++;

		if (sniff_flag >= 2 || (cmp_error && (sniff_flag >= 1)))
		{

			fprintf(stderr, "\t\tAccelerator number %d ", i+1);
			timeprt(begin_time);
			fprintf(stderr, ", record %d", type74->accelerator[i].index);
			if (cmp_error) fprintf(stderr, "<=== ERROR\n");
			else fprintf(stderr, "\n");

		}
	}

	/* recover network info for SEED version 2.3 */
        if (type10.version >= 2.3)
        {
                type74->network_code = parse_nchar (&blockette_ptr, 2);
        }
        else
        {
                type74->network_code = "";
        }


/*                 +=======================================+                 */
/*=================| link the type74 into the type74 table |=================*/
/*                 +=======================================+                 */

	append_linklist_element (type74, type74_head, type74_tail);

}
