/*===========================================================================*/
/* SEED reader     |              process_stnh             |  station header */
/*===========================================================================*/
/*
	Name:		process_stnh
	Purpose:	recover station header data and put into 
				globally-available storage
	Usage:		void process_stnh ();
			process_stnh ();
	Input:		none
	Output:		none
	Externals:	input - defined in structures.h, allocated in globals.h
	Warnings:	unknown blockette type encountered
	Errors:		none
	Called by:	main program
	Calls to:	Routines to parse the various blockette types:
			parse_type50 - Station ID Blockette parser
			parse_type51 - Station Comment Blockette parser
			parse_type52 - Channel ID Blockette parser
			parse_type53 - Response (Poles and Zeroes) Blockette parser
			parse_type54 - Response (Coefficients) Blockette parser
			parse_type55 - Response (List) Blockette parser
			parse_type56 - Generic Response Blockette parser
			parse_type57 - Decimation Blockette parser
			parse_type58 - Channel Sensitivity Blockette parser
			parse_type59 - Channel Comment Blockette parser
			parse_type60 - Response Reference Blockette parser
	Algorithm:	determine which type of blockette is encountered; parse it into the appropriate table

	Notes:		some of the parsing routines and their respective print routines are untested as of the initial release date; there were no blockettes of those types available as of the cited date.
	Problems:	none known
	References:	Halbert et al, 1988; see main routine
	Language:	C, hopefully ANSI standard
	Author:		Dennis O'Neill
	Revisions:	07/15/88  Dennis O'Neill  Initial preliminary release 0.9
			11/21/88  Dennis O'Neill  Production release 1.0
			05/02/90  Sue Schoch      Included blockette 57 support. Rel 2.0
                	05/15/90  Sue Schoch      Included blockette 60 support.
			03/23/99  Stephane Zuzlewski	Included blockette 62 support.
*/

#include "rdseed.h"

void process_stnh ()
{
	char *input_data_ptr;			/* pointer to input data */

	input_data_ptr = input.data;

/*                 +=======================================+                 */
/*=================|        Process station headers        |=================*/
/*                 +=======================================+                 */

	blockette_length = 0;
	blockette_type = -1;

	while (blockette_type != 0)
	{
		input_data_ptr += blockette_length;

		read_blockette (&input_data_ptr, blockette, &blockette_type, 
			&blockette_length);


                if (blockette_type != 0)
                        blockette_sanity_checker(blockette, 
						input.recordnumber, 
						input.recordnumber + 1, sniff_flag);

		if (blockette_type == 0)
			continue;

		switch (blockette_type)
		{
			case 50:
				parse_type50 (blockette);
				break;

			case 51: 
                                parse_type51 (blockette);
                                break;
                        case 52: 
                                parse_type52 (blockette);
                                break;
                        case 53: 
                                parse_type53 (blockette);
                                break;
                        case 54: 
                                parse_type54 (blockette);
                                break;
                        case 55: 
                                parse_type55 (blockette);
                                break;
                        case 56: 
                                parse_type56 (blockette);
                                break;
                        case 57: 
                                parse_type57 (blockette);
                                break;
                        case 58: 
                                parse_type58 (blockette);
                                break;
                        case 59: 
                                parse_type59 (blockette);
                                break;
                        case 60: 
                                parse_type60 (blockette);
                                break;

			case 61:
				parse_type61(blockette);
				break;
			case 62:
				parse_type62(blockette);
				break;
			default:

				fprintf (stderr, "WARNING (process_stationh):  ");
                        	fprintf (stderr, "unknown blockette type %d found in record %ld.\n",
                                		blockette_type, input.recordnumber);

                        	fprintf (stderr, "\tExecution continuing.\n");
				break;
		}

	}

/*                 +=======================================+                 */
/*=================|                cleanup                |=================*/
/*                 +=======================================+                 */

}
