#include "sanity.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "rdseed.h"
#include <ctype.h>
#include <errno.h>

extern int relax_flag;

int is_a_number(/*char*/);
int seconds_to_beginning_of_year(/*int*/);



/*  This function will check the length of a specified blockette
    to make sure that it is within the bounds of the max_blockette_length
    variable. If not, it adds an error to the result structure */

void check_length(field, field_number, field_name, field_length, blockette_length,
		  min_blockette_length, max_blockette_length, result)
char field[];
int field_number;
char field_name[];
int field_length;
int blockette_length;
int min_blockette_length;
int max_blockette_length;
struct result_struct *result;
{
    char error_string[256];

    strncpy (error_string, "\0", 128);

    /* before we can add an error to the result struct, we must first
       make an error string. This string will be printed out when
       all of the errors for the blockette will be printed */

    if (blockette_length < min_blockette_length) {
        sprintf(error_string, "Blockette is too short. It has length %i bytes, while length should be %i bytes or more", blockette_length, min_blockette_length);
	add_error_to_result(field, field_number, field_name, field_length, 0, error_string, result);
    }
    else if (max_blockette_length > 0 && blockette_length > max_blockette_length) {
        sprintf(error_string, "Blockette is too long. It has length %i bytes, while length should be %i bytes or less", blockette_length, max_blockette_length);
        add_error_to_result(field, field_number, field_name, field_length, 0, error_string, result);
    }

    return;
}



/*  This function will check the field to the mask that the field has
    to conform to as specified in the SEED Format V2.3 manual. Many variables
    are passed to this function. The vaiable names should be obvious. If a 
    decimal is required, the position should be passed, same goes if an
    exponential is required. If a number is signed, the signed variable should
    be one. If a certian aspect is not required (ie decimal or exponential), the
    variable should be set to zero. If the field does not meet the requirements,
    an error is added to the result struct */

int check_number_mask(field, field_number, field_name, field_length, 
				decimal_position, exponential_position, 
				is_signed, result) 

char field[];
int field_number;
char field_name[];
int field_length;
int decimal_position;
int exponential_position;
int is_signed;
struct result_struct *result;

{
    char error_string[256];
    int is_field_correct = 1;
    int i;
    int number_not_encountered;
    int number_encountered;

    strncpy(error_string, "\0", 128);


    /* check to see if decimal is present and in proper position */
    if (decimal_position && !relax_flag) {
        if (field[decimal_position - 1] != '.') {
	    sprintf(error_string, "There is no decimal point at position %i in the field", decimal_position);
	    add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	    is_field_correct = 0;
	}
    }

    /* check to see if the exponential is present. Also check character 
       after exponential to make sure it is legal */
    if (exponential_position) {
        if (field[exponential_position - 1] != 'e' && field[exponential_position - 1] != 'E') {
	    sprintf(error_string, "There is no exponential at position %i in the field", exponential_position);
	    add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	    is_field_correct = 0;
	}
	if (field[exponential_position] != '+' && field[exponential_position] != '-' && !is_a_number(field[exponential_position]) && field[exponential_position] != ' ') {
	    sprintf(error_string, "Character after the exponential 'e' is not a space, a number, +, or -, it is a '%c'", field[exponential_position]);
	    add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	    is_field_correct = 0;
	}
    }

    /* check to see if sign is present (it could also be the case that there is no sign) */
    if (is_signed) {
        for (i=0; i < field_length; ++i) {
  	    if (field[i] == '+' || field[i] == '-' || is_a_number(field[i])) {
	      /* the three lines below used to be commented out */
 	      /*  sprintf(error_string, "Character in first position in the field is not a space, a number, +, or -, it is a %c", field[0]); 
 	        add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result); 
 	        is_field_correct = 0; */
	        break;
	    }
	}
    }

    number_not_encountered = 1;
    number_encountered = 0;

    /* make sure that there are no characters in the number field which shouldn't be there */
    for (i=0; i < field_length; ++i) {
        if (field[i] == '\0') {
	    sprintf(error_string, "The NULL character does not belong at position %i in the field", field[i], i + 1);
	    add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	    is_field_correct = 0;
	    continue;
	}
        if (is_signed && (field[i] == '+' || field[i] == '-') && number_not_encountered) {
	    continue;
	}
	if (decimal_position && i == decimal_position - 1) {
	    continue;
	}

	if (relax_flag && (field[i] == '.'))
		continue;
  
	if (exponential_position && i == exponential_position - 1) {
	    number_not_encountered = 1;
	    number_encountered = 0;
	    continue;
	}
	if (exponential_position && i >= exponential_position && (field[i] == '+' || field[i] == '-' || field[i] == ' ') && number_not_encountered) {
	    continue;
	}
	if (field[i] == ' ' && number_not_encountered) {
	    continue;
	}	
	if (is_a_number(field[i])) {
	    number_not_encountered = 0;
	    number_encountered = 1;
	}
	else {
	    sprintf(error_string, "The character '%c' does not belong at position %i in the field", field[i], i + 1);
	    add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	    is_field_correct = 0;
	}
    }
	
	
    return(is_field_correct);
}



/*  this function checks to make sure that the field is between two acceptable numbers 
    This function also assumes that check_number_mask has been called first */

int check_number_between_two_values(field, field_number, field_name, field_length, min_value, max_value, result)
char field[];
int field_number;
char field_name[];
int field_length;
double min_value;
double max_value;
struct result_struct *result;
{
    char error_string[256];
    char field_text[128];
    
    int i;
    int error = 0;

    double field_value;


    strncpy(field_text, field, field_length);
    field_text[field_length] = '\0';

    /* since spaces are legal, they must be converted to zeors */
    for (i=0; i < field_length; ++i) {
        if (field_text[i] == ' ') {
	    field_text[i] = '0';
	}
    }
    
    field_value = atof(field_text);

    /*    printf("filed_value: %f, min_value: %f, max_value: %f\n", field_value, min_value, max_value);*/

    /* the if statements below handle the possible floating point and 
       integer combinations that the field, the min_value, and the
       max_value may take on */

    if (field_value < min_value || field_value > max_value) {
      if ((double)((int)(field_value)) == field_value && (double)(int)(max_value) == max_value && (double)(int)(min_value) == min_value) {
        sprintf(error_string, "The value %i is outside of the range of accepted values (%i to %i) for this field", (int)(field_value), (int)(min_value), (int)(max_value));
        add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	error = 1;
      }
      else if ((double)((int)(field_value)) == field_value && (double)(int)(max_value) == max_value) {
        sprintf(error_string, "The value %i is outside of the range of accepted values (%f to %i) for this field", (int)(field_value), min_value, (int)(max_value));
        add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	error = 1;
      }
      else if ((double)((int)(field_value)) == field_value && (double)(int)(min_value) == min_value) {
        sprintf(error_string, "The value %i is outside of the range of accepted values (%i to %f) for this field", (int)(field_value), (int)(min_value), max_value);
        add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	error = 1;
      }
      else if ((double)((int)(min_value)) == min_value && (double)(int)(max_value) == max_value) {
        sprintf(error_string, "The value %f is outside of the range of accepted values (%i to %i) for this field", field_value, (int)(min_value), (int)(max_value));
        add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	error = 1;
      }
      else if ((double)((int)(field_value)) == field_value) {
        sprintf(error_string, "The value %i is outside of the range of accepted values (%f to %f) for this field", (int)(field_value), min_value, max_value);
        add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	error = 1;
      }
      else {
        sprintf(error_string, "The value %f is outside of the range of accepted values (%f to %f) for this field", field_value, min_value, max_value);
        add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	error = 1;
      }
    }

    return(error);
}



/*  This function checks to make sure that the field is less than a given value.
    This function also assumes that the field has been checked with check_number_mask first */

void check_number_less_than_value(field, field_number, field_name, field_length, max_value, result)
char field[];
int field_number;
char field_name[];
int field_length;
double max_value;
struct result_struct *result;
{
    char error_string[256];
    char field_text[128];

    int i;

    double field_value;


    strncpy(field_text, field, field_length);
    field_text[field_length] = '\0';

    /* because spaces are legal, we must convert them to zeros */
    for (i=0; i < field_length; ++i) {
        if (field_text[i] == ' ') {
	    field_text[i] = '0';
	}
    }
    
    field_value = atof(field_text);

    /* the if statements below handle whether or not we are dealing
       with a floating point number or an integer */

    if (field_value >= max_value) {
      if ((double)((int)(field_value)) == field_value && (double)(int)(max_value) == max_value) {
        sprintf(error_string, "The value %i is greater than or equal to the maximum value (%i) allowed for this field", (int)(field_value), (int)(max_value));
        add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
      }
      else {
        sprintf(error_string, "The value %f is greater than or equal to the maximum value (%f) allowed for this field", field_value, max_value);
        add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
      }
    }

    return;
}



/*  This function checks to see if the field is greater than the minimum accepted value.
    It also assumes that check_number_mask has been called first */

void check_number_greater_than_value(field, field_number, field_name, field_length, min_value, result)
char field[];
int field_number;
char field_name[];
int field_length;
double min_value;
struct result_struct *result;
{
    char error_string[256];
    char field_text[128];

    int i;

    double field_value;


    strncpy(field_text, field, field_length);
    field_text[field_length] = '\0';

    /* since spaces are legal, we must convert them to zeros */
    for (i=0; i < field_length; ++i) {
        if (field_text[i] == ' ') {
	    field_text[i] = '0';
	}
    }
    
    field_value = atof(field_text);

    /* the if statements below handle whether or not we are dealing
       with a floating point number or an integer */

    if (field_value <= min_value) {
      if ((double)((int)(field_value)) == field_value && (double)(int)(min_value) == min_value) {
        sprintf(error_string, "The value %i is less than or equal to the minimum value (%i) allowed for this field", (int)(field_value), (int)(min_value));
        add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
      }
      else {
        sprintf(error_string, "The value %f is less than or equal to the minimum value (%f) allowed for this field", field_value, min_value);
        add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
      }
    }

    return;
}



/*  This function checks to see if the field is equal to the required value for the field.
    It also assumes that check_number_mask has been called first */

/*  if the user wants this function to print an error, they should set print_error to 1,
    if they would like to handle any conditions that arise, they should set it to 0.
    This function will return 0 if the expected number is not equal to the actual, or 1 if it is.
    This feature is useful when cascading these function calls to see if a value is one
    of several */

int check_number_equal_to_value(field, field_number, field_name, field_length, equal_value, print_error, result)
char field[];
int field_number;
char field_name[];
int field_length;
double equal_value;
int print_error;
struct result_struct *result;
{
    char error_string[256];
    char field_text[128];

    int i;
    int number_ok = 0;                        /* assume that expected does not equal to actual */

    double field_value;


    strncpy(field_text, field, field_length);
    field_text[field_length] = '\0';

    /* because spaces are legal, they must be converted to zeros */
    for (i=0; i < field_length; ++i) {
        if (field_text[i] == ' ') {
	    field_text[i] = '0';
	}
    }
    
    field_value = atof(field_text);

    if (field_value != equal_value) {
        number_ok = 0;

        /* the if statements below handle whether or not we are dealing
           with a floating point number or an integer */

	if (print_error) {
	  if ((double)((int)(equal_value)) == equal_value) {
	    sprintf(error_string, "The value %i is not equal to the value that is set for this field (%i)", (int)(field_value), (int)(equal_value));
	    add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	  }
	  else {
	    sprintf(error_string, "The value %f is not equal to the value that is set for this field (%f)", field_value, equal_value);
	    add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	  }
	}
    }
    else {
        number_ok = 1;
    }

    return(number_ok);
}



/*  This function makes sure that the field is equal to one of two required values for the field.
    It assumes that check_number_mask has been called first */

void check_number_one_of_two_values(field, field_number, field_name, field_length, value1, value2, result)
char field[];
int field_number;
char field_name[];
int field_length;
double value1;
double value2;
struct result_struct *result;
{
    char error_string[256];
    char field_text[128];

    int i;

    double field_value;


    /*printf("value1 in check_number... is: %f\n", value1);*/
    strncpy(field_text, field, field_length);
    field_text[field_length] = '\0';

    for (i=0; i < field_length; ++i) {
        if (field_text[i] == ' ') {
	    field_text[i] = '0';
	}
    }
    
    field_value = atof(field_text);

    /* the if statements below handle whether or not we are dealing
       with a floating point number or an integer */

    if (field_value != value1 && field_value != value2) {
      if ((double)((int)(value1)) == value1 && (double)((int)(value2)) == value2) {
        sprintf(error_string, "The value %i is not equal to one of the two accepted values for this field (%i and %i)", (int)(field_value), (int)(value1), (int)(value2));
        add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
      }
      else {
        sprintf(error_string, "The value %f is not equal to one of the two accepted values for this field (%f and %f)", field_value, value1, value2);
        add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
      }
    }

    return;
}



/*  This function returns the number that is in the given field 
    It assumes that the number it is to return is an integer */

int return_number(field, field_length)
char field[];
int field_length;
{
    char field_text[128];

    strncpy (field_text, "\0", 128);

    
    strncpy (field_text, field, field_length);

    return(atoi(field_text));
}



/*  This function checks the field to make sure that it meets the format requirements
    for a time field. Requirements are defined in the SEED v2.3 Format manual */

int check_time_mask(field, field_number, field_name, max_field_length, result)
char field[];
int field_number;
char field_name[];
int max_field_length;
struct result_struct *result;
{
    int i, j;
    int field_length;
    int error_occured = 0;

    char field_text[80];
    char error_string[256];

    char year_char[5];
    char day_char[4];
    char hour_char[3];
    char minute_char[3];
    char second_char[3];
    char frac_second_char[5];

    int year_int;
    int day_int;
    int hour_int;
    int minute_int;
    int second_int;
    int frac_second_int;

    strncpy(year_char, "\0", 5);
    strncpy(day_char, "\0", 4);
    strncpy(hour_char, "\0", 3);
    strncpy(minute_char, "\0", 3);
    strncpy(second_char, "\0", 3);
    strncpy(frac_second_char, "\0", 5);


    /* find the actual field length (could be less than max_field_length */
    for (i=0; i < max_field_length; ++i) {
	field_text[i] = field[i];

	if (field[i] == '~') {
	    break;
	}
    }
    field_text[i + 1] = '\0';

    field_length = i + 1;

    if (i == max_field_length && field[i] != '~') {
        sprintf(error_string, "There is no '~' terminator at the end of this variable length TIME field");
        add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
    }


    /* search through field to make sure that all field separators are present and correct 
       This section also makes sure that there are only numbers in the fields */
    for (j=0; j < i; ++j) {
        if (j == 4 && field_text[j] != ',') {
	    sprintf(error_string, "There is no ',' between the year and the day in the TIME field");
            add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	    error_occured = 1;
	}
	if (j == 8 && field_text[j] != ',') {
	    sprintf(error_string, "There is no ',' between the day and the hour in the TIME field");
            add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	    error_occured = 1;
	}
	if (j == 11 && field_text[j] != ':') {
	    sprintf(error_string, "There is no ':' between the hour and the minute in the TIME field");
            add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	    error_occured = 1;
	}
	if (j == 14 && field_text[j] != ':') {
	    sprintf(error_string, "There is no ':' between the minute and the second in the TIME field");
            add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	    error_occured = 1;
	}
	if (j == 17 && field_text[j] != '.') {
	    sprintf(error_string, "There is no '.' between the second and fractions of a second in the TIME field");
            add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	    error_occured = 1;
	}

	/* if we are at a field separator, skip testing to make sure that it is a number */
	if (j == 4 || j == 8 || j == 11 || j == 14 || j == 17) {
	    continue;
	}
   
	if (field_text[j] == '\0') {
	    sprintf(error_string, "There should not be a NULL at position %i in the TIME field", j + 1);
            add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	    error_occured = 1;
	    continue;
	}

        if (!is_a_number(field_text[j])) {
	    sprintf(error_string, "There should not be a '%c' at position %i in the TIME field", field_text[j], j + 1);
            add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	    error_occured = 1;
	}
    }


    /* If an error was not found, separate field so we can check to see if numbers are reasonable */
    if (!error_occured) {
        for (j=0; j < 4  && j <= i && field_text[j] != '~'; ++j) {
	    year_char[j] = field_text[j];
	}
	for (j=5; j < 8 && j <= i && field_text[j] != '~'; ++j) {
	    day_char[j - 5] = field_text[j];
	}
	for (j=9; j < 11 && j <= i && field_text[j] != '~'; ++j) {
	    hour_char[j - 9] = field_text[j];
	}
	for (j=12; j < 14 && j <= i && field_text[j] != '~'; ++j) {
	    minute_char[j - 12] = field_text[j];
	}
	for (j=15; j < 17 && j <= i && field_text[j] != '~'; ++j) {
	    second_char[j - 15] = field_text[j];
	}
	for (j=18; j < 22 && j <= i && field_text[j] != '~'; ++j) {
	    frac_second_char[j - 18] = field_text[j];
	}
    
	if (year_char[0] != '\0') {
	    year_int = atoi(year_char);

	    /* the following check is to 9999 because sometimes, when networks do not know the
	       end time of the record, they write years such as 4000 */
	    if (year_int < 1900 || year_int > 9999) {
	        sprintf(error_string, "The year %s might have a problem, it is not between 1900 and 9999", year_char);
                add_error_to_result(field, field_number, field_name, field_length, 0, error_string, result);
	    }
	}
	if (day_char[0] != '\0') {
	    day_int = atoi(day_char);
	    if ((year_int % 4 == 0 && year_int % 100 != 0) || year_int % 400 == 0) {
  	        if (day_int < 0 || day_int > 366) {
		    sprintf(error_string, "The day %s is not between 000 and 366", day_char);
                    add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
		}
	    }
	    else {
	        if (day_int == 366) {
		    sprintf(error_string, "366 is not a legal day in a non-leap year");
                    add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
		}
		else if (day_int < 0 || day_int > 365) {
		    sprintf(error_string, "The day %s is not between 000 and 365", day_char);
                    add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
		}
	    }
	}
	if (hour_char[0] != '\0') {
	    hour_int = atoi(hour_char);
	    if (hour_int < 0 || hour_int > 23) {
	        sprintf(error_string, "The hour %s is not between 00 and 23", hour_char);
                add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	    }
	}
	if (minute_char[0] != '\0') {
	    minute_int = atoi(minute_char);
	    if (minute_int < 0 || minute_int > 59) {
	        sprintf(error_string, "The minute %s is not between 00 and 59", minute_char);
                add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	    }
	}
	if (second_char[0] != '\0') {
	    second_int = atoi(second_char);
	    if (second_int == 60) {
	        sprintf(error_string, "The second is set to 60, please make sure that this is a valid leap second");
		add_error_to_result(field, field_number, field_name, field_length, 0, error_string, result);
	    }
	    else if (second_int < 0 || second_int > 59) {
	        sprintf(error_string, "The second %s is not between 00 and 59", second_char);
                add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	    }
	}
	if (frac_second_char[0] != '\0') {
	    frac_second_int = atoi(frac_second_char);
	    if (frac_second_int < 0 || frac_second_int > 9999) {
	        sprintf(error_string, "The fraction of a second %s is not between 0000 and 9999", frac_second_char);
                add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	    }
	}
    }

    
    return(i + 1);
}



/*  This function makes sure that time1 occurs earlier than time2.
    It also assumes that check_time_mask has been called for both time1 and time2 

    To check the times, this function finds the number of seconds since 1970
    in order that it can perform a simple integer comparison */

void check_two_times(time1_field, time1_field_number, time1_name, time1_length, 
		     time2_field, time2_field_number, time2_name, time2_length, result)
char time1_field[];
int time1_field_number;
char time1_name[];
int time1_length;
char time2_field[];
int time2_field_number;
char time2_name[];
int time2_length;
struct result_struct *result;
{
    int i;

    char time1_text[25];
    char time2_text[25];
    char error_string[256];

    char year1_char[5];
    char day1_char[4];
    char hour1_char[3];
    char minute1_char[3];
    char second1_char[3];
    
    int year1;
    int day1;
    int hour1;
    int minute1;
    int second1;

    char year2_char[5];
    char day2_char[4];
    char hour2_char[3];
    char minute2_char[3];
    char second2_char[3];
 
    int year2;
    int day2;
    int hour2;
    int minute2;
    int second2;

    int seconds_since_first_of_year_1 = 0;
    int seconds_since_first_of_year_2 = 0;

    unsigned long int seconds_since_1970_1 = 0;
    unsigned long int seconds_since_1970_2 = 0;

    strncpy(year1_char, "\0", 5);
    strncpy(day1_char, "\0", 4);
    strncpy(hour1_char, "\0", 3);
    strncpy(minute1_char, "\0", 3);
    strncpy(second1_char, "\0", 3);

    strncpy(year2_char, "\0", 5);
    strncpy(day2_char, "\0", 4);
    strncpy(hour2_char, "\0", 3);
    strncpy(minute2_char, "\0", 3);
    strncpy(second2_char, "\0", 3);


    --time1_length;                 /* do not include ~ terminator in count */
    --time2_length;

    strncpy(time1_text, time1_field, time1_length);
    strncpy(time2_text, time2_field, time2_length);


    /* separate individual fields from time field1 and 2 */
    for (i=0; i < time1_length && i < 4; ++i) {
        year1_char[i] = time1_field[i];
    }
    for (i=5; i < time1_length && i < 8; ++i) {
        day1_char[i - 5] = time1_field[i];
    }
    for (i=9; i < time1_length && i < 11; ++i) {
        hour1_char[i - 9] = time1_field[i];
    }
    for (i=12; i < time1_length && i < 14; ++i) {
        minute1_char[i - 12] = time1_field[i];
    }
    for (i=15; i < time1_length && i < 17; ++i) {
        second1_char[i - 15] = time1_field[i];
    }

    for (i=0; i < time2_length && i < 4; ++i) {
        year2_char[i] = time2_field[i];
    }
    for (i=5; i < time2_length && i < 8; ++i) {
        day2_char[i - 5] = time2_field[i];
    }
    for (i=9; i < time2_length && i < 11; ++i) {
        hour2_char[i - 9] = time2_field[i];
    }
    for (i=12; i < time2_length && i < 14; ++i) {
        minute2_char[i - 12] = time2_field[i];
    }
    for (i=15; i < time2_length && i < 17; ++i) {
        second2_char[i - 15] = time2_field[i];
    }
    
    year1 = atoi(year1_char);
    day1 = atoi(day1_char);
    hour1 = atoi(hour1_char);
    minute1 = atoi(minute1_char);
    second1 = atoi(second1_char);

    if (time2_length != 0) {
        year2 = atoi(year2_char);
	day2 = atoi(day2_char);
	hour2 = atoi(hour2_char);
	minute2 = atoi(minute2_char);
	second2 = atoi(second2_char);
    }
    else {
        return;
    }


    /* convert fields to an actual number that can be compared with each other */
    seconds_since_first_of_year_1 = (60 * 60 * 24 * day1) + ((hour1 * 60 * 60) + (minute1 * 60) + second1);
    seconds_since_1970_1 = seconds_to_beginning_of_year(year1) + seconds_since_first_of_year_1; 

    seconds_since_first_of_year_2 = (60 * 60 * 24 * day2) + ((hour2 * 60 * 60) + (minute2 * 60) + second2);
    seconds_since_1970_2 = seconds_to_beginning_of_year(year2) + seconds_since_first_of_year_2; 

    if (seconds_since_1970_2 < seconds_since_1970_1) {
        sprintf(error_string, "The %s (%s) is earlier \n\t\tthan the %s (%s)", time2_name, time2_text, time1_name, time1_text);
        add_error_to_result(time1_field, time1_field_number, time1_name, time1_length + 1, 1, error_string, result);
    }
	/* commented out because event detect cause this */
    /*else if (seconds_since_1970_2 == seconds_since_1970_1) {
        sprintf(error_string, "The %s (%s) is at the same \n\t\ttime as the %s (%s)", time2_name, time2_text, time1_name, time1_text);
        add_error_to_result(time1_field, time1_field_number, time1_name, time1_length + 1, 1, error_string, result);
    }*/


    return;
}



/*  This function makes sure that the field that is passed to it meets the criteria that are
    imposed upon it by the restrinctions variable. the restirnctions variable should just
    be a string that matches the restrictions that are associated with the field as specified
    in the SEED v2.3 format manual. If the field is variable length, set variable_length to 1,
    if it must be a certain amount of characters, set variable_length to 0. */

int check_string_mask(field, field_number, field_name, max_field_length, variable_length, restrictions, result)
char field[];
int field_number;
char field_name[];
int max_field_length;
int variable_length;
char restrictions[];
struct result_struct *result;
{
    int i;
    int field_length;

    /* start out assuming that nothing is allowed, then change if discovered otherwise */
    int upper_case = 0;
    int lower_case = 0;
    int digits = 0;
    int punctuation = 0;
    int spaces = 0;
    int underline = 0;

    int something_encountered = 0;
    int space_not_encountered = 1;

    char error_string[256];


    /* figure out what restrictions the field has */
    for (i=0; i < 6; ++i) {
        if (restrictions[i] == '\0') {
	    break;
	}
	
	switch (restrictions[i]) {
	case 'U':
	    upper_case = 1;
	    break;
	case 'L':
	    lower_case = 1;
	    break;
	case 'N':
	    digits = 1;
	    break;
	case 'P':
	    punctuation = 1;
	    underline = 1;
	    break;
	case 'S':
	    spaces = 1;
	    break;
	case '_':
	    underline = 1;
	    break;
	}
    }

    /* if the restrictions field is NULL, then assume everyting is allowed */
    if (i == 0) {
        upper_case = 1;
	lower_case = 1;
	digits = 1;
	punctuation = 1;
	spaces = 1;
	underline = 1;
    }

    if (variable_length) {
        /* find the field length, because it could be different from the max_field_length allowed */
        for (i=0; i < max_field_length; ++i) {
	    if (field[i] == '~') {
	        break;
	    }
	}
	field_length = i + 1;

	if (i == max_field_length && field[i] != '~') {
	    sprintf(error_string, "There is no '~' terminator at the end of this variable length string field");
	    add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	}
    }
    else {
        field_length = max_field_length;
    }

    if (variable_length != 0) {
        variable_length = 1;
    }

    /* check to see that none of the restrictions are violated in the field */
    for (i = 0; i < field_length - variable_length; ++i) {
        if (field[i] == '\0') {
	    sprintf(error_string, "The NULL character at position %i in the field is not permitted", field[i], i + 1);
            add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	}

        if (!upper_case && isupper(field[i]) && space_not_encountered) {
	    sprintf(error_string, "The upper-case character '%c' at position %i in the field is not permitted", field[i], i + 1);
            add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	}
	else if (upper_case && isupper(field[i]) && !space_not_encountered && !variable_length) {
	    sprintf(error_string, "The upper-case character '%c' at position %i in the field is not permitted \n\t\tafter a space has been encountered", field[i], i + 1);
            add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	}
	else if (upper_case && isupper(field[i]) && space_not_encountered) 
	    something_encountered = 1;
        if (!lower_case && islower(field[i]) && space_not_encountered) {
	    sprintf(error_string, "The lower-case character '%c' at position %i in the field is not permitted", field[i], i + 1);
            add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	}
	else if (lower_case && islower(field[i]) && !space_not_encountered && !variable_length) {
	    sprintf(error_string, "The lower-case character '%c' at position %i in the field is not permitted \n\t\tafter a space has been encountered", field[i], i + 1);
            add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	}
	else if (lower_case && islower(field[i]) && space_not_encountered)
	    something_encountered = 1;
        if (!digits && isdigit(field[i]) && space_not_encountered) {
	    sprintf(error_string, "The digit '%c' at position %i in the field is not permitted", field[i], i + 1);
            add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	}
	else if (digits && isdigit(field[i]) && !space_not_encountered && !variable_length) {
	    sprintf(error_string, "The digit '%c' at position %i in the field is not permitted \n\t\tafter a space has been encountered", field[i], i + 1);
            add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	}
	else if (digits && isdigit(field[i]) && space_not_encountered) 
	    something_encountered = 1;

        if (!underline && field[i] == '_' && space_not_encountered) {
	    sprintf(error_string, "The underscore character '%c' at position %i in the field is not permitted", field[i], i + 1);
            add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	}
	else if (underline && field[i] == '_' && !space_not_encountered && !variable_length) {
	    sprintf(error_string, "The underscore character '%c' at position %i in the field is not permitted \n\t\tafter a space has been encountered", field[i], i + 1);
            add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	}
	else if (underline && field[i] == '_' && space_not_encountered) {
	    something_encountered = 1;
	    continue;
	}
        if (!punctuation && ispunct(field[i]) && space_not_encountered) {
	    sprintf(error_string, "The punctuation character '%c' at position %i in the field is not permitted", field[i], i + 1);
            add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	}
	else if (punctuation && ispunct(field[i]) && !space_not_encountered && !variable_length) {
	    sprintf(error_string, "The punctuation character '%c' at position %i in the field is not permitted \n\t\tafter a space has been encountered", field[i], i + 1);
            add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	}
	else if (punctuation && ispunct(field[i]) && space_not_encountered) 
	    something_encountered = 1;
        if (!spaces && isspace(field[i])) {
	    if (variable_length == 0 && space_not_encountered) {
	        space_not_encountered = 0;
	    }
	    else if (space_not_encountered == 0) {
	        ;
	    }
	    else {
	        sprintf(error_string, "The space at position %i in the field is not permitted", i + 1);
		add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	    }
	}
    }

/*     if (something_encountered == 0) { */
/*         sprintf(error_string, "The field is blank"); */
/* 	add_error_to_result(field, field_number, field_name, field_length, 0, error_string, result); */
/*     } */


    
    return(field_length);
}



/*  This function checks to see if the field is equal to one of two character values */
/*  WARNING!!! this function only works on the character in the first position of the field */

void check_char_one_of_two_values(field, field_number, field_name, field_length, value1, value2, result)
char field[];
int field_number;
char field_name[];
int field_length;
char value1;
char value2;
struct result_struct *result;
{
    char error_string[256];
    char field_text[128];

    char field_value;


    strncpy(field_text, field, field_length);
    field_text[field_length] = '\0';
    
    field_value = field_text[0];

    if (field_value != value1 && field_value != value2) {
        sprintf(error_string, "The character %c is not one of the two accepted characters for this field (%c and %c)", field_value, value1, value2);
        add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
    }

    return;
}



/*  This function checks to see if the field is equal to the required field value */
/*  If print_error is non zero, the error will be printed, else it will return if the
    epected value equaled the actual value or not */
/*  WARNING!!! this function will only check the first character of the field */

int check_char_value(field, field_number, field_name, field_length, value, print_error, result)
char field[];
int field_number;
char field_name[];
int field_length;
char value;
int print_error;
struct result_struct *result;
{
    char error_string[256];
    char field_text[128];

    int char_ok = 0;                    /* assume the expected and actual are different */

    char field_value;


    strncpy(field_text, field, field_length);
    field_text[field_length] = '\0';

    field_value = field_text[0];

    if (field_value != value) {
        char_ok = 0;
	if (print_error) {
	    sprintf(error_string, "The character %c is not the same as the accepted character for this field (%c)", field_value, value);
	    add_error_to_result(field, field_number, field_name, field_length, 1, error_string, result);
	}
    }
    else {
        char_ok = 1;
    }

    return(char_ok);
}


/* function to print out if the correct number of bytes were not read */
void print_blockette_length_problem(bytes_read, num_bytes_to_read, result)
int bytes_read;
int num_bytes_to_read;
struct result_struct *result;
{
    fprintf(stderr, "Warning!! -- Blockette %i, %s, has a problem\n", result->blockette_number, result->blockette_name);

    fprintf(stderr, "\tOnly %i bytes read, while %i bytes should have been read\n\n", bytes_read, num_bytes_to_read);

	result->error = 1;
    return;
}



/* This function will add an error to the linked list of error_structs that resides in the result struct */

void add_error_to_result(field, field_number, field_name, field_length, error, error_string, result)
char field[];
int field_number;
char field_name[];
int field_length;
int error;
char error_string[];
struct result_struct *result;
{
    struct error_struct *new_error;
    struct error_struct *current_link;

    int i;
    int number_of_nulls = 0;

    char field_text[128];
    char field_string_text[256];

    strncpy(field_text, "\0", 128);


    /* copy the field to the error_struct so the user can see what went wrong */
    for (i=0; i < field_length; ++i) {
        if (field[i] == '\0') {
	    strcat(field_text, "NULL");
	    number_of_nulls++;
	}
	else {
	    field_text[i + (number_of_nulls * 4)] = field[i];
	}
    }
    sprintf(field_string_text, "Field %i (%s) looks like: '%s'", field_number, field_name, field_text);

    /* allocate a new error_struct */
    new_error = (struct error_struct *)malloc(sizeof(struct error_struct));
    if (new_error == NULL) {
        result->mem_alloc_error = 1;
        return;
    }
/*     else { */
/*         fprintf(stderr, "Memory at %x allocated\n", new_error); */
/*     } */


    /* set the new error_struct to all the values that it needs to have */
    new_error->field_number = field_number;
    strncpy(new_error->field_string, field_string_text, 256);
    strncpy(new_error->error_string, error_string, 256);
    new_error->error = error;
    new_error->next = NULL;

    /* get to the end of the linked list so we can append this new error_struct to it */
    current_link = result->next;
    while (result->next != NULL && current_link->next != NULL) {
	current_link = current_link->next;
    }

    /* append new error_struct to the end of the linked list */
    if (result->next == NULL) {
        result->next = new_error;
    }
    else {
        current_link->next = new_error;
    }


    /* set result->error to the proper value
       0 is no errors or warnings
       1 is ONLY warnings have occured
       2 is ONLY errors have occured
       3 is BOTH errors and warnings have occured */

    /* error will be 0 if it was a warning, 1 if an error */
    if (error) {
        switch (result->error) {
        case 0:
	case 2:
            result->error = 2;
	    break;
	case 1:
	case 3:
	    result->error = 3;
	    break;
	}
    }
    else {
        switch (result->error) {
	case 0:
	case 1:
	    result->error = 1;
	    break;
	case 2:
	case 3:
	    result->error = 3;
	    break;
	}
    }

    return;
}
	
        

/*  This function will print out the linked list that is contained in the result struct.
    It is this linked list which contains a listing of all the errors that were contained 
    in the blockettes */

void print_errors(result, verbose_option)
struct result_struct *result;
int verbose_option;
{
    int error_printed = 0;
    char mem_string[128];
    struct error_struct *current_link;
    struct error_struct *copy_of_link;


    current_link = result->next;
    
    /* begin to go through the linked list printing all nodes on te list */
    while (current_link != NULL) {
      if (verbose_option >= 3 || !error_printed) {
        if (current_link->error == 0) {
	    fprintf(stderr, "WARNING!! ");
	}
	else if (current_link->error == -10) {
	    fprintf(stderr, "TEST ");
	}
	else if (verbose_option == 0)
	    ;
	else {
	    fprintf(stderr, "ERROR!! ");
	}
      }
      if (verbose_option >= 3 || (verbose_option == 2 && !error_printed)) {
	if (result->start_record_number == result->end_record_number) {
	    fprintf(stderr, "-- Record %i, Blockette %i, %s, field %i\n", result->start_record_number, result->blockette_number, result->blockette_name, current_link->field_number);
	}
	else if (verbose_option != 0) {
	    fprintf(stderr, "-- Record %i to %i, Blockette %i, %s, field %i\n", result->start_record_number, result->end_record_number, result->blockette_number, result->blockette_name, current_link->field_number);
	}
	if (verbose_option != 0) {
	    fprintf(stderr, "\t%s\n", current_link->field_string);
	    fprintf(stderr, "\tThe error is: %s\n\n", current_link->error_string);
	}

	if (verbose_option < 3) {
		fprintf(stderr, "Blockette %d\n", result->blockette_number);
 
		fprintf(stderr, "%s\n%s\n",
			current_link->field_string,
			current_link->error_string);

	  error_printed = 1;
	}
      }
      else if (!error_printed) {
	if (verbose_option != 0) {
	  if (result->start_record_number == result->end_record_number) {
	    fprintf(stderr, "-- Record %i, Blockette %i, %s, field %i\n", result->start_record_number, result->blockette_number, result->blockette_name, current_link->field_number);
	  }
	  else {
	    fprintf(stderr, "-- Record %i to %i, Blockette %i, %s, field %i\n", result->start_record_number, result->end_record_number, result->blockette_number, result->blockette_name, current_link->field_number);
	  }
	}
	fprintf(stderr, "Warning: Blockette %d\n", result->blockette_number);
 
	fprintf(stderr, "%s\n%s\n",
			current_link->field_string,
			current_link->error_string);

	error_printed = 1;
      }

      copy_of_link = current_link;
      current_link = current_link->next;
      free(copy_of_link);
    }

    result->next = NULL;

    return;
}
        


/* this funciton figures out if a character represents a digit */
int is_a_number(letter)
char letter;
{
    if (letter < 48 || letter > 57) {
        return(0);
    }
    else {
        return(1);
    }
}



/* This function figures out how many second there have been since 1/1/1970 until 1/1/year */

int seconds_to_beginning_of_year (year)
int year;
{
  int seconds_since_1970 = 0;
  int seconds_in_year;
  int days_in_year;

  int year_divided_by_4;
  int year_divided_by_100;
  int year_divided_by_400;
  int i;

  for (i=1970; i < year; ++i) {

    /* this section sees if a year is a leap year.  If it is, it sets the
       number of days to 366, if not, it sets it to 365 */
    year_divided_by_4 = i % 4;
    year_divided_by_100 = i % 100;
    year_divided_by_400 = i % 400;

    if ( (year_divided_by_4 == 0 && year_divided_by_100 != 0) || year_divided_by_400 == 0 )
      days_in_year = 366;
    else
      days_in_year = 365;

    seconds_in_year = 60 * 60 * 24 * days_in_year;
    seconds_since_1970 = seconds_since_1970 + seconds_in_year;
  }

  return (seconds_since_1970);
}



/* this function will return the length of the variable
   length field in the field variable. It will look for
   a ~ until it hits max_length */

int get_variable_field(blockette, field, max_length)
char blockette[];
char *field;
int max_length;
{
  char *temp_pointer;
  int i;

  strncpy(field, "\0", max_length + 1);
  temp_pointer = strstr(blockette, "~");

  if (temp_pointer - blockette <= max_length) {  /* pointer arithmetic */
    /*strcpy(field, strtok(blockette, "~"));    I would have used this, but strtok did not behave 
                                                right when the tok was the in the first position of the array */
    /*for (i=0; i < temp_pointer - blockette; ++ i)
      field[i] = blockette[i];*/
    i = temp_pointer - blockette;
    strncpy(field, blockette, i);
    field[strlen(field)] = '~';
  }
  else {
    strncpy(field, blockette, max_length);
  }

  return(strlen(field));
}
